import asyncio
import io
import os
import warnings
from datetime import datetime, timedelta
import base64
import pyperclip
import pyautogui
from pywinauto.application import Application
from rich.console import Console

from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.models.dto.rpa_sistema_dto import RpaSistemaDTO
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.api.client import download_file_from_historico, get_config_by_name, send_file
from worker_automate_hub.utils.util import (
    create_temp_folder,
    delete_folder,
    kill_all_emsys,
    login_emsys,
    save_pdf_emsys, 
    type_text_into_field, 
    worker_sleep,
    set_variable,
    )

ASSETS_BASE_PATH = 'assets/fidc/'
console = Console()



async def retorno_cobranca(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    '''
        Processo de retorno de cobrança e retorno de cobrança extraodinaria do FIDC
    '''
    if 'extraordinario' in task.nomProcesso:
        BASE_FILE_NAME = f'COB_001_240_3576383_{datetime.now().strftime("%y%m%d")}_N.ret'
    else:
        BASE_FILE_NAME = f'COB_001_240_3576383_{datetime.now().strftime("%y%m%d")}_00.ret'
    
    try:
        nome_usuario = os.environ.get('USERNAME') or os.environ.get('USER')
        nome_pasta = f"{nome_usuario}_arquivos"
        #Setando tempo de timeout
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)
        #Delete temp folder
        await delete_folder(nome_pasta)
        #Cria Pasta temporaria
        temp_folder = await create_temp_folder()
        console.print(f"Pasta criada com sucesso. \n {temp_folder}", style="bold green")
        try:
            downloaded_files =  await download_file_from_historico(task.historico_id)
        except Exception as ex:
            log_msg = f"Erro ao baixar o arquivo: {str(ex)}"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
            #Cria arquivo
        for file in downloaded_files:
            if file['tipo'].lower() == 'ret':
                decode = base64.b64decode(file['conteudo'])
                with open(os.path.join(temp_folder, BASE_FILE_NAME), 'wb') as f:
                    f.write(decode)
                break
        #Pega Config para logar no Emsys
        config = await get_config_by_name("login_emsys")
        folders_paths = await get_config_by_name("Folders_Fidc")
    
        #Abre um novo emsys
        await kill_all_emsys()
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field('Retorno de Cobranca', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            await worker_sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Retorno de Cobranca' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login
        
        await worker_sleep(5)

        #Digita Cobrança
        app = Application(backend='win32').connect(title="Processa Arquivo de Retorno de Cobrança")
        window_ret_cob = app['Processa Arquivo de Retorno de Cobrança']
        campo = window_ret_cob.child_window(class_name='TDBIEditCode')
        campo.set_focus()
        campo.type_keys('4')
        campo.type_keys('{TAB}')

        await worker_sleep(3)

        #Seleciona '0027'
        app = Application(backend='win32').connect(title="Busca Cobrança Banco")
        window_cob_banco = app['Busca Cobrança Banco']
        window_cob_banco.set_focus()
        for _ in range(3):
            window_cob_banco.type_keys('{DOWN}')
            await worker_sleep(1)
        window_cob_banco.type_keys('{ENTER}')
        
        await worker_sleep(5)

        #Digita caminho do arquivo
        app = Application(backend='win32').connect(title="Processa Arquivo de Retorno de Cobrança")
        window_ret_cob = app['Processa Arquivo de Retorno de Cobrança']
        file_path = window_ret_cob.child_window(class_name='TDBIEditString')
        file_path.set_text(f"{temp_folder}\\{BASE_FILE_NAME}")

        await worker_sleep(5)

        #Uncheck "Deduzir automaticamente taxa de cobrança"
        taxa_cobranca = window_ret_cob.child_window(class_name='TCheckBox', found_index=2)
        taxa_cobranca.click()

        await worker_sleep(5)

        #Alterar data da liquidação check
        taxa_cobranca = window_ret_cob.child_window(class_name='TCheckBox', found_index=1)
        taxa_cobranca.click()

        await worker_sleep(5)

        #Processar retorno
        console.print("Processando retorno")
        process_return = window_ret_cob.child_window(class_name='TBitBtn', found_index=4)
        process_return.set_focus()
        process_return.click()
        
        await worker_sleep(30)

        #Informar data de liquidação
        app = Application(backend='win32').connect(title="Information")
        window_info = app['Information']
        btn_ok = window_info.child_window(class_name='TButton')
        btn_ok.click()
        pyautogui.click(959, 558)

        await worker_sleep(10)

        #Confirmar Arquivo
        console.print("Confirmando Arquivo")
        app = Application(backend='win32').connect(title="Processa Arquivo de Retorno de Cobrança")
        window_ret_cob = app['Processa Arquivo de Retorno de Cobrança']
        btn_confirm = window_ret_cob.child_window(class_name='TBitBtn', found_index=2)
        btn_confirm.click()
        pyautogui.click(1146, 387)
        
        await worker_sleep(5)

        #Alterar Data de liquidação
        app = Application(backend='win32').connect(title="Retorno de Cobrança")
        window_ret_cob = app['Retorno de Cobrança']
        field_date = window_ret_cob.child_window(class_name='TDBIEditDate')
        btn_ok = window_ret_cob.child_window(title='&OK', class_name='TBitBtn')
        field_date.set_focus()
        field_date.type_keys(datetime.now().strftime("%d/%m/%Y"))
        # field_date.type_keys("11/01/2025")
        await worker_sleep(1)
        pyautogui.click(907, 566)

        await worker_sleep(5)

        #Confrima Alterar Data de liquidação
        try:
            app = Application(backend='win32').connect(title="Confirm")
            window_info = app['Confirm']
            pyautogui.click(917, 566)
        except Exception as ex:
            log_msg = f"Não houve tela de confirmação de alteração da data: {str(ex)}"
            logger.error(log_msg)
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            ) 
        

        await worker_sleep(5)

        #Registros confirmados
        try:
            app = Application(backend='win32').connect(title="Information")
            window_info = app['Information']
            pyautogui.click(952, 561)
        except Exception as ex:
            log_msg = f"Não houve tela para confirmar registros: {str(ex)}"
            logger.error(log_msg)
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(20)
        
        i = 0
        while i > 10:
            try:
                app = Application(backend='win32').connect(title="Warning")
                window_warning = app['Warning']
                window_warning.child_window(title='&No', class_name='TBitBtn').click()
                pyautogui.click(999,561)
            except Exception as ex:
                console.print("Sem cleintes bloqueados", style="bold green")
                break
            
        #Clica em "No" novamente por garantia
        pyautogui.click(999,561)
        await worker_sleep(5)

        #Imprimir
        try:
            app = Application(backend='win32').connect(title="Processa Arquivo de Retorno de Cobrança")
            window_ret_cob = app['Processa Arquivo de Retorno de Cobrança']
            pyautogui.click(1007, 375)
        except Exception as ex:
            log_msg = f"Não clicou em imprimir: {str(ex)}"
            logger.error(log_msg)
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(5)

        try:
            app = Application(backend="win32").connect(title="Print Preview")
        except Exception as ex:
            log_msg = f"Tela de prin preview não foi aberta corretamente: {str(ex)}"
            logger.error(log_msg)
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        
        await worker_sleep(5)

        await save_pdf_emsys(temp_folder + "\\" + BASE_FILE_NAME.replace(".ret", ".pdf"))

        await worker_sleep(5)

        try:
            with open(temp_folder + "\\" + BASE_FILE_NAME.replace(".ret", ".pdf"), 'rb') as file:
                file_bytes = io.BytesIO(file.read())
            # Enviando o arquivo para o backoffices
            await send_file(task.historico_id, BASE_FILE_NAME.replace(".ret", ""), "pdf", file_bytes, file_extension="pdf")
        except Exception as ex:
            log_msg = f"Erro ao enviar o arquivo para o backoffices: {str(ex)}"
            logger.error(log_msg)
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await delete_folder(nome_pasta)

        log_msg = f"Sucesso ao executar retorno de cobrança"
        logger.error(log_msg)
        console.print(log_msg, style="bold green")
        return RpaRetornoProcessoDTO(
            sucesso=True, retorno=log_msg, status=RpaHistoricoStatusEnum.Sucesso, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )
    except Exception as ex:
        log_msg = f"Erro Processo Retorno Cobranca: {str(ex)}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )



import asyncio
import re
from datetime import datetime
import getpass
import os
import warnings
import time
import uuid
import win32clipboard
import difflib
import pyautogui
import pyperclip
from pywinauto.keyboard import send_keys
from PIL import Image, ImageEnhance
import pytesseract
from pywinauto.application import Application
from pywinauto_recorder.player import set_combobox
from rich.console import Console

from worker_automate_hub.api.client import (
    get_config_by_name,
    get_status_nf_emsys,
    sync_get_config_by_name,
)
from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    check_nota_importada,
    cod_icms,
    error_after_xml_imported,
    import_nfe,
    importar_notas_outras_empresas,
    is_window_open,
    is_window_open_by_class,
    itens_not_found_supplier,
    kill_all_emsys,
    login_emsys,
    rateio_despesa,
    select_documento_type,
    set_variable,
    tipo_despesa,
    type_text_into_field,
    verify_nf_incuded,
    warnings_after_xml_imported,
    worker_sleep,
    zerar_icms,
)
from worker_automate_hub.utils.utils_nfe_entrada import EMSys

pyautogui.PAUSE = 0.5
console = Console()

emsys = EMSys()


async def entrada_de_notas_39(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        # Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)

        console.print(config)

        # Seta config entrada na var nota para melhor entendimento
        nota = task.configEntrada
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # Fecha a instancia do emsys - caso esteja aberta
        await kill_all_emsys()

        #Verifica se a nota ja foi lançada
        console.print("\nVerifica se a nota ja foi lançada...")
        nf_chave_acesso = int(nota.get("nfe"))
        status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
        if status_nf_emsys.get("status") == "Lançada":
            console.print("\nNota ja lançada, processo finalizado...", style="bold green")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Nota já lançada",
                status=RpaHistoricoStatusEnum.Descartado,
            )
        else:
            console.print("\nNota não lançada, iniciando o processo...")
        
        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field(
                "Nota Fiscal de Entrada", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(1)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(10)

        # Procura campo documento
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        document_type = await select_documento_type(
            "NOTA FISCAL DE ENTRADA ELETRONICA - DANFE"
        )
        if document_type.sucesso == True:
            console.log(document_type.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=document_type.retorno,
                status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(10)

        # Clica em 'Importar-Nfe'
        imported_nfe = await import_nfe()
        if imported_nfe.sucesso == True:
            console.log(imported_nfe.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=imported_nfe.retorno,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(10)

        # Move até 'Notas de Outras Empresas'
        pyautogui.press('down', presses=5)
        # Clica em  'OK' para selecionar
        pyautogui.click(970, 666)
        await worker_sleep(3)

        try:

            #INTERAGINDO COM A TELA DE NOTAS DE OUTRAS EMPRESAS
            app = Application().connect(class_name="TFrmImportarNotaOutraEmpresa")
            main_window = app["TFrmImportarNotaOutraEmpresa"]
            console.print("A tela de Importar Nota de Outra Empresa foi encontrada!")

            console.print('Inserindo o codigo da empresa...')
            empresa = str(int(nota.get("cnpjFornecedor")[8:12]))
            
            edit = main_window.child_window(
                class_name="TDBIEditCode", found_index=0
            )
            edit.set_edit_text(empresa)
            
            await worker_sleep(1)
            
            pyautogui.hotkey('tab')

            console.print('Inserindo a data de emissão da nota')
            dt_emissao = nota.get("dataEmissao")
            numero_nota = nota.get("numeroNota")

            try:
                data_formatada = datetime.fromisoformat(dt_emissao[:-6]).strftime("%d/%m/%Y")
            except ValueError:
                data_formatada = dt_emissao
            
            edit = main_window.child_window(
                class_name="TDBIEditDate", found_index=0
            )
            edit.set_edit_text(data_formatada)
            edit = main_window.child_window(
                class_name="TDBIEditDate", found_index=1
            )
            edit.set_edit_text(data_formatada)

            console.print('Inserindo o numero da nota')
            edit = main_window.child_window(
                class_name="TDBIEditString", found_index=0
            )
            edit.set_edit_text(numero_nota)

            await worker_sleep(2)

            pesquisar_full_path = "assets\\entrada_notas\\PesquisarNFOutrasEmpresas.png"
            try:
                button_location = pyautogui.locateCenterOnScreen(pesquisar_full_path, confidence=0.6)
                if button_location:
                    pyautogui.click(button_location)
                    console.print("Botão 'Pesquisar' clicado com sucesso!")
            except pyautogui.ImageNotFoundException:
                window_rect = main_window.rectangle()
                console.print(f"Area que sera utulizada para o screenshot {window_rect}...\n")

                try:
                    button_location = pyautogui.locateCenterOnScreen(pesquisar_full_path, region=(window_rect.left, window_rect.top, window_rect.width(), window_rect.height()))
                    if button_location:
                        button_location = (button_location.x + window_rect.left, button_location.y + window_rect.top)
                        console.print(f"Botão encontrado nas coordenadas: {button_location}")
                        pyautogui.click(button_location)
                except pyautogui.ImageNotFoundException:
                    console.print("Erro - Botão Pesquisar na tela de Importação de Outras Empresas não foi encontrado após tentar capturar a tela")
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno="Erro - Botão Pesquisar na tela de Importação de Outras Empresas não foi encontrado após tentar capturar a tela",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
            except Exception as e:
                console.print(
                    f"Não foi possivel seguir pois não foi fois possivel interagir com o botão de Pesquisar na tela Importação de Outras Empresas,Error: {e}...\n"
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Não foi possivel seguir pois não foi fois possivel interagir com o botão de Pesquisar na tela Importação de Outras Empresas,Error: {e}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
            
            i = 0
            max_attempts = 17

            while i < max_attempts:
                i += 1
                console.print("Verificando se a nota foi encontrada...\n")
                try:
                    main_window.set_focus()
                    no_data_full_path = "assets\\entrada_notas\\no_data_display.png"
                    img_no_data = pyautogui.locateCenterOnScreen(no_data_full_path, confidence=0.6)
                    if img_no_data:
                        console.print("'No data display' ainda aparente. Tentando novamente...")
                        await worker_sleep(10)
                except pyautogui.ImageNotFoundException:
                    console.print("'No data display' não encontrado na tela!")
                    break

                except Exception as e:
                    console.print(f"Ocorreu um erro: {e}")


            console.print(f"Clicando em Importar")
            pyautogui.press('tab', presses=2)
            importar_btn_full_path = "assets\\entrada_notas\\importar_nf_outras_empresa.png"
            try:
                button_location = pyautogui.locateCenterOnScreen(importar_btn_full_path, confidence=0.6)
                if button_location:
                    pyautogui.moveTo(button_location)
                    await worker_sleep(1)
                    pyautogui.click(button_location)
                    console.print("Botão 'Importar' clicado com sucesso!")
                    await worker_sleep(1)
                    
                    attempts = 0
                    max_attempts = 7
                    while attempts < max_attempts:
                        attempts += 1
                        try:
                            button_location = pyautogui.locateCenterOnScreen(importar_btn_full_path, confidence=0.6)
                            if button_location:
                                pyautogui.moveTo(button_location)
                                await worker_sleep(20)
                                pyautogui.click(button_location)
                                console.print("Botão 'Importar' ainda persiste, clicando...")
                        except pyautogui.ImageNotFoundException:
                            console.print("Botão 'Importar' não encontrado, saindo...")
                            break
            except pyautogui.ImageNotFoundException:
                    window_rect = main_window.rectangle()
                    console.print(f"Area que sera utlizada para o screenshot {window_rect}...\n")

                    try:
                        button_location = pyautogui.locateCenterOnScreen(importar_btn_full_path, region=(window_rect.left, window_rect.top, window_rect.width(), window_rect.height()))
                        if button_location:
                            button_location = (button_location.x + window_rect.left, button_location.y + window_rect.top)
                            console.print(f"Botão encontrado nas coordenadas: {button_location}")
                            pyautogui.click(button_location)
                    except pyautogui.ImageNotFoundException:
                        console.print("Erro - Botão Importar na tela de Importação de Outras Empresas não foi encontrado após tentar capturar a tela")
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno="Erro - Botão Importar na tela de Importação de Outras Empresas não foi encontrado após tentar capturar a tela",
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                        )
            except Exception as e:
                console.print(
                    f"Não foi possivel seguir pois não foi fois possivel interagir com o botão de Importar na tela Importação de Outras Empresas,Error: {e}...\n"
                )
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Não foi possivel seguir pois não foi fois possivel interagir com o botão de Importar na tela Importação de Outras Empresas,Error: {e}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )


            await worker_sleep(10)
            # VERIFICANDO A EXISTENCIA DE ERRO
            erro_pop_up = await is_window_open("Erro")
            if erro_pop_up["IsOpened"] == True:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="Nota não encontrada no momento da importação",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
            

            max_attempts = 10
            i = 0
            while i < max_attempts:
                information_pop_up = await is_window_open("Informações para importação da Nota Fiscal Eletrônica")
                if information_pop_up["IsOpened"] == True:
                    break
                else:
                    console.print(f"Aguardando a tela Informações para importação da Nota Fiscal Eletrônica...\n")
                    await worker_sleep(3)
                    i += 1
            
            if i >= max_attempts:
                error_work = await error_after_xml_imported()
                return RpaRetornoProcessoDTO(
                    sucesso=error_work.sucesso,
                    retorno=error_work.retorno,
                    status=error_work.status,
                    tags=error_work.tags
                )

            console.print(
                f"Marcando Manter calculo PIS/COFINS...\n"
            )
            app = Application(backend="uia").connect(title="Informações para importação da Nota Fiscal Eletrônica")
            main_window = app["Informações para importação da Nota Fiscal Eletrônica"]
            try:
                checkbox = main_window.child_window(
                    title="Manter cálculo PIS/COFINS",
                    class_name="TCheckBox",
                    control_type="CheckBox",
                )
            except:
                checkbox = main_window.child_window(
                    title=re.compile(".*PIS/COFINS.*"),
                    class_name="TCheckBox",
                    control_type="CheckBox"
                )
            if not checkbox.get_toggle_state() == 1:
                checkbox.click()
                console.print("Realizado com sucesso... \n")

            if nota.get("existeDespesa") != "Não":
                app = Application().connect(
                title="Informações para importação da Nota Fiscal Eletrônica"
                )
                main_window = app["Informações para importação da Nota Fiscal Eletrônica"]

                # INTERAGINDO COM A DATA DE ENTRADA
                await worker_sleep(2)
                try:
                    recebimento_fisico = nota.get("recebimentoFisico", None)
                    if recebimento_fisico:
                        recebimento_fisico = nota["recebimentoFisico"].split(" ")
                        pyautogui.write(recebimento_fisico[0])
                        await worker_sleep(2)
                except:
                    console.print(
                        f"A chave recebimentoFisico não está presente na config de entrada...\n"
                    )


                combo_box_natureza_operacao = main_window.child_window(
                            class_name="TDBIComboBox", found_index=0
                        )
                combo_box_natureza_operacao.click()
                await worker_sleep(4)
                set_combobox("||List", "1152-ENTRADA P/TRANSFERENCIA MERCADORIA S/EST")
                await worker_sleep(3)

                pyautogui.hotkey("tab")

                despesa = "291"
                tipo_despesa_work = await tipo_despesa(despesa)
                if tipo_despesa_work.sucesso == True:
                    console.log(tipo_despesa_work.retorno, style="bold green")
                else:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=tipo_despesa_work.retorno,
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )

                # INTERAGINDO COM O CAMPO Manter Natureza de Operação selecionada
                console.print(
                    f"Selecionando a opção 'Manter Natureza de Operação selecionada'...\n"
                )
                checkbox = main_window.child_window(
                    title="Manter Natureza de Operação selecionada",
                    class_name="TDBICheckBox",
                )
                if checkbox.exists():
                    try:
                        if not checkbox.get_toggle_state() == 1:
                            checkbox.click()
                            console.print(
                                "A opção 'Manter Natureza de Operação selecionada' selecionado com sucesso... \n"
                            )
                    except AttributeError as e:
                        console.print(f"Erro ao verificar o estado: {e}")
                        console.print("Tentando uma alternativa...")

                        properties = checkbox.get_properties()
                        if properties.get("ToggleState", 0) != 1:
                            checkbox.click()
                            console.print(
                                "A opção 'Manter Natureza de Operação selecionada' foi selecionada com sucesso com método alternativo... \n"
                            )
                else:
                    console.print("Checkbox não encontrado ou inacessível.")

            max_attempts = 3
            i = 0
            while i < max_attempts:
                console.print("Clicando no botão de OK...\n")
                try:
                    try:
                        btn_ok = main_window.child_window(title="Ok")
                        btn_ok.click()
                    except:
                        pyautogui.click(1093,728)
                except:
                    console.print("Não foi possivel clicar no Botão OK... \n")

                await worker_sleep(2)
                i += 1
        except:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao importar nota fiscal.",
                status=RpaHistoricoStatusEnum.Descartado,
            )

        await worker_sleep(80)
        
        await emsys.verify_warning_and_error("Information", "No")
        
        try:
            console.print("Verificando itens não localizados ou NCM...\n")
            itens_by_supplier = await is_window_open_by_class("TFrmAguarde", "TMessageForm")

            if itens_by_supplier["IsOpened"] == True:
                itens_by_supplier_work = await itens_not_found_supplier(nota.get("nfe"))

                if not itens_by_supplier_work.sucesso:
                    return itens_by_supplier_work

        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao verificar a existência de POP-UP de itens não localizados: {error}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(3)
        try:
            console.print(f"Trabalhando com itens com multiplas referencias.\n")
            itens_nao_semelhantes = []
            # TRABALHANDO COM ITENS QUE POSSUEM MULTIPLAS REFERÊNCIAS
            while True:
                await worker_sleep(3)
                console.print(
                    f"Verificando a existencia de fornecedor com múltiplas referencias...\n"
                )
                itens_by_supplier = await is_window_open_by_class(
                    "TFrmSelecionaItensFornecedor", "TFrmSelecionaItensFornecedor"
                )
                if itens_by_supplier["IsOpened"] == True:
                    new_app = Application(backend="uia").connect(
                        class_name="TFrmSelecionaItensFornecedor"
                    )
                    window = new_app["TFrmSelecionaItensFornecedor"]
                    window.set_focus()
                    await worker_sleep(2)

                    console.print(f"Obtendo item com multiplas referências...\n")
                    console.print(f"Tirando print da janela para realização do OCR...\n")

                    text_captured = False
                    count_while = 0
                    max_attempts = 3
                    item_da_nota = ''

                    while count_while < max_attempts:
                        window_rect = window.rectangle()
                        console.print(f"Area que sera utulizada para o screenshot {window_rect}...\n")
                        screenshot = window.capture_as_image()

                        username = getpass.getuser()
                        short_uuid = str(uuid.uuid4()).replace('-', '')[:6]
                        path_to_png = f"C:\\Users\\{username}\\Downloads\\{short_uuid}.png"
                        screenshot.save(path_to_png)
                        while not os.path.exists(path_to_png) or os.path.getsize(path_to_png) == 0:
                            time.sleep(0.1)
                        console.print(f"Print salvo em {path_to_png}...\n")

                        await worker_sleep(2)
                        console.print("Preparando a imagem para maior resolução e assertividade no OCR...\n")
                        image = Image.open(path_to_png)
                        image = image.convert("L")
                        enhancer = ImageEnhance.Contrast(image)
                        image = enhancer.enhance(2.0)
                        image.save(path_to_png)
                        console.print("Imagem preparada com sucesso...\n")

                        console.print(f"Imagem antes do OCR: {image}\n")
                        console.print(f"Dimensões da imagem: {image.size}\n")

                        console.print("Realizando OCR...\n")
                        captured_text = pytesseract.image_to_string(image)
                        console.print(f"Texto Full capturado {captured_text}, tentando obter o item da nota...\n")

                        match = re.search(r"Item da Nota:\s*(.*)\s*", captured_text)
                        if os.path.exists(path_to_png):
                            os.remove(path_to_png)
                            console.print(f"Imagem apagada com sucesso do diretorio {path_to_png}... \n")
                        else:
                            console.print(f"Imagem não encontrada para realização do OCR... \n")

                        console.print(f"Texto extraido do RegEx: {match}... \n")
                        if match:
                            item_da_nota = match.group(1).strip()
                            console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                            text_captured = True
                            break
                        else:
                            if match:
                                item_da_nota = match.group(1).strip()
                                console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                                text_captured = True
                                break
                            else:
                                match = re.search(r"Item da (Nota|Nata|N0ta)\s*(.*)\s*", captured_text)
                                if match:
                                    item_da_nota = match.group(1).strip()
                                    console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                                    text_captured = True
                                    break
                                else:
                                    console.print(f"Tentativa {count_while + 1} de {max_attempts} falhou. Tentando novamente...\n")
                                    count_while += 1

                    if not text_captured:
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno="Quantidade de tentativa atingida (3), não foi possivel capturar o item da nota com multiplas referencias para andamento no processo",
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                        )

                    console.print(
                        f"Interagindo com os multiplos itens e detectando qual ira ser selecionado para andamento no processo... \n"
                    )
                    copied_codes_list = []
                    count = 0
                    last_copied = None

                    while True:
                        if count == 0:
                            send_keys("^({HOME})")
                        else:
                            send_keys("{DOWN 1}")

                        send_keys("^c")
                        win32clipboard.OpenClipboard()
                        window_message = win32clipboard.GetClipboardData().strip()
                        win32clipboard.CloseClipboard()

                        console.print(f"Linha copiada: {window_message}... \n")

                        if last_copied is not None and window_message == last_copied:
                            break

                        copied_codes_list.append(
                            {"linha": count, "conteudo": window_message}
                        )

                        last_copied = window_message
                        count = +1

                    console.print(f"Valores copiados {copied_codes_list}... \n")

                    extracted_items = {}

                    for entry in copied_codes_list:
                        content = entry["conteudo"]
                        lines = content.split("\r\n")
                        for line in lines:
                            items = line.split("\t")
                            if len(items) >= 3:
                                cod_item = items[0].strip()
                                descricao = items[1].strip()
                                cod_barra = items[2].strip()

                                extracted_items[cod_item] = {
                                    "Descrição": descricao,
                                    "Código de Barra": cod_barra,
                                    "Linha": entry["linha"],
                                }

                    best_match = None
                    highest_ratio = 0.0

                    for cod_item, info in extracted_items.items():
                        descricao = info["Descrição"]
                        similarity_ratio = difflib.SequenceMatcher(
                            None, item_da_nota, descricao
                        ).ratio()

                        if similarity_ratio > highest_ratio:
                            highest_ratio = similarity_ratio
                            best_match = {
                                "Cod Item": cod_item,
                                "Descrição": descricao,
                                "Código de Barra": info["Código de Barra"],
                                "Linha": info["Linha"],
                                "Similaridade": similarity_ratio,
                            }

                    if best_match and highest_ratio > 0.7:
                        console.print(
                            f"Melhor semelhança encontrada {best_match}, selecionando... \n"
                        )
                        send_keys("^({HOME})")
                        send_keys("{DOWN " + str(best_match["Linha"]) + "}")
                        send_keys("%o")
                        await worker_sleep(2)
                    else:
                        itens_nao_semelhantes.append(
                            {"item": item_da_nota, "match": best_match}
                        )
                        send_keys("%o")
                else:
                    console.print(
                        "Não possui a tela de fornecedor com múltiplas referencias, seguindo com o processo...\n"
                    )
                    break

            if len(itens_nao_semelhantes) > 0:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Não foi possivel encontrar o item mais proximo ao item da nota com multiplas referencias {itens_nao_semelhantes}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
        except Exception as error:
            console.print("Erro durante a trativa de multiplas referencias, erro : {error}")

        # VERIFICANDO A EXISTENCIA DE ERRO
        erro_pop_up = await is_window_open("Erro")
        if erro_pop_up["IsOpened"] == True:
            error_work = await error_after_xml_imported()
            return RpaRetornoProcessoDTO(
                sucesso=error_work.sucesso,
                retorno=error_work.retorno,
                status=error_work.status,
                tags=error_work.tags
            )

        # Seleciona pagamento
        console.log("Seleciona Pagamento", style="bold yellow")
        pyautogui.click(623, 374)
        await worker_sleep(1)
        pyautogui.click(889, 349)
        await worker_sleep(1)
        pyautogui.write("27")
        await worker_sleep(1)
        pyautogui.hotkey("enter")

        # Digita "Valor"
        pyautogui.click(1285, 352)
        await worker_sleep(1)
        pyautogui.hotkey("ctrl", "a")
        pyautogui.hotkey("del")
        await worker_sleep(1)
        pyautogui.write(nota["valorNota"])
        await worker_sleep(3)

        await emsys.incluir_registro()
        await worker_sleep(25)

        console.print(
            "Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n"
        )
        itens_variacao_maxima = await is_window_open_by_class(
            "TFrmTelaSelecao", "TFrmTelaSelecao"
        )
        if itens_variacao_maxima["IsOpened"] == True:
            app = Application().connect(class_name="TFrmTelaSelecao")
            main_window = app["TFrmTelaSelecao"]
            send_keys("%o")


        await worker_sleep(5)
        aviso_pop_up = await is_window_open("Aviso")
        if aviso_pop_up["IsOpened"] == True:
            await emsys.verify_warning_and_error("Aviso", "OK")
            try:
                pyautogui.click(623, 374)
                await worker_sleep(1)
                pyautogui.press('home')
                console.print("Alterando a NOP...\n")

                app = Application().connect(class_name="TFrmNotaFiscalEntrada", timeout=30)
                main_window = app["TFrmNotaFiscalEntrada"]
                main_window.set_focus()

                select_box_nop_select = main_window.child_window(class_name="TDBIComboBox", found_index=0)
                select_box_nop_select.click()
                pyautogui.write("1152")
                await worker_sleep(2)
                pyautogui.hotkey("enter")
                await worker_sleep(2)

                max_try =  10
                i = 0
                first_nop = False
                while i <= max_try:
                    select_box_nop_select = main_window.child_window(class_name="TDBIComboBox", found_index=0)
                    nop_selected = select_box_nop_select.window_text()
                    if '1152 - ENTRADAS P/ TRANSFERENCIA DE MERCADORIAS- 1.152' == nop_selected and first_nop == False:
                        first_nop = True
                    else:
                        if '1152 - ENTRADAS P/ TRANSFERENCIA DE MERCADORIAS- 1.152' == nop_selected:
                            break
                    
                    pyautogui.press('down')
                    await worker_sleep(2)
                    i = i + 1


                if i <= max_try and first_nop:
                    console.print(f"A segunda correspondência '1152 - ENTRADAS P/ TRANSFERENCIA DE MERCADORIAS- 1.152' foi encontrada e selecionada.")
                else:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro não foi encontrada a segunda correspondência dentro do número máximo de tentativas ({max_try})",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
                
                await emsys.incluir_registro()
                await worker_sleep(25)

                console.print("Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n")
                itens_variacao_maxima = await is_window_open_by_class("TFrmTelaSelecao", "TFrmTelaSelecao")
                if itens_variacao_maxima["IsOpened"] == True:
                    app = Application().connect(class_name="TFrmTelaSelecao")
                    main_window = app["TFrmTelaSelecao"]
                    send_keys("%o")
                    await worker_sleep(3)
                
                aviso_pop_up = await is_window_open("Aviso")
                if aviso_pop_up["IsOpened"] == True:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro na validação de CFOP, foi encontrado mais de uma opção com a mesma informação",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                    )
                
                console.print("Verificando a existencia de Warning informando que a Soma dos pagamentos não bate com o valor da nota. ...\n")
                warning_pop_up_pagamentos = await is_window_open("Warning")
                if warning_pop_up_pagamentos["IsOpened"] == True:
                    return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"A soma dos pagamentos não bate com o valor da nota.",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
                
                await worker_sleep(60)
                console.print("\nVerifica se a nota ja foi lançada...")
                nf_chave_acesso = int(nota.get("nfe"))
                status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
                if status_nf_emsys.get("status") == "Lançada":
                    return RpaRetornoProcessoDTO(
                        sucesso=True,
                        retorno="Nota Lançada com sucesso!",
                        status=RpaHistoricoStatusEnum.Sucesso,
                    )
            except Exception as e:
                select_box_nop_select.click()
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro ao alterar o NOP, erro {e}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

        try:
            max_attempts = 7
            i = 0
            aguarde_rateio_despesa = True

            while i < max_attempts:
                await worker_sleep(3)

                from pywinauto import Desktop

                for window in Desktop(backend="uia").windows():
                    if "Rateio" in window.window_text():
                        aguarde_rateio_despesa = False
                        console.print(
                            "A janela 'Rateio da Despesas' foi encontrada. Continuando para andamento do processo...\n"
                        )
                        break

                if not aguarde_rateio_despesa:
                    break

                i += 1

            if aguarde_rateio_despesa:
                console.log(f"Número máximo de tentativas atingido. A tela para Rateio da Despesa não foi encontrada.")

            if not aguarde_rateio_despesa:
                despesa_rateio_work = await rateio_despesa(nota.get("filialEmpresaOrigem"))
                if despesa_rateio_work.sucesso == True:
                    console.log(despesa_rateio_work.retorno, style="bold green")
                    await emsys.verify_warning_and_error("Information", "OK")
                    await emsys.verify_warning_and_error("Informação", "OK")
                    await worker_sleep(15)
                else:
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=despesa_rateio_work.retorno,
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=despesa_rateio_work.tags
                    )
        except Exception as error:
            console.print(f"Falha ao realizar rateio {error}")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao realizar rateio {error}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )


        warning_pop_up = await is_window_open("Warning")
        if warning_pop_up["IsOpened"] == True:
            app = Application().connect(title="Warning")
            main_window = app["Warning"]
            main_window.set_focus()

            console.print(f"Obtendo texto do Warning...\n")
            console.print(f"Tirando print da janela do warning para realização do OCR...\n")

            window_rect = main_window.rectangle()
            screenshot = pyautogui.screenshot(
                region=(
                    window_rect.left,
                    window_rect.top,
                    window_rect.width(),
                    window_rect.height(),
                )
            )
            username = getpass.getuser()
            path_to_png = f"C:\\Users\\{username}\\Downloads\\warning_popup_{nota.get("nfe")}.png"
            screenshot.save(path_to_png)
            console.print(f"Print salvo em {path_to_png}...\n")

            console.print(
                f"Preparando a imagem para maior resolução e assertividade no OCR...\n"
            )
            image = Image.open(path_to_png)
            image = image.convert("L")
            enhancer = ImageEnhance.Contrast(image)
            image = enhancer.enhance(2.0)
            image.save(path_to_png)
            console.print(f"Imagem preparada com sucesso...\n")
            console.print(f"Realizando OCR...\n")
            captured_text = pytesseract.image_to_string(Image.open(path_to_png))
            console.print(
                f"Texto Full capturado {captured_text}...\n"
            )
            os.remove(path_to_png)
            if 'movimento não permitido' in captured_text.lower():
                return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Filial: {nota.get("filialEmpresaOrigem")} está com o livro fechado ou encerrado, verificar com o setor fiscal",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
            else:
                return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Warning não mapeado para seguimento do robo, mensagem: {captured_text}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

        await worker_sleep(4)
        console.print(
            "Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n"
        )
        itens_variacao_maxima = await is_window_open_by_class(
            "TFrmTelaSelecao", "TFrmTelaSelecao"
        )
        if itens_variacao_maxima["IsOpened"] == True:
            app = Application().connect(class_name="TFrmTelaSelecao")
            main_window = app["TFrmTelaSelecao"]
            send_keys("%o")
            await worker_sleep(7)
        
        try:
            console.print("Verificando itens não localizados ou NCM...\n")
            itens_by_supplier = await is_window_open_by_class("TFrmAguarde", "TMessageForm")

            if itens_by_supplier["IsOpened"] == True:
                itens_by_supplier_work = await itens_not_found_supplier(nota.get("nfe"))

                if not itens_by_supplier_work.sucesso:
                    return itens_by_supplier_work

        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao verificar a existência de POP-UP de itens não localizados: {error}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        
        # VERIFICANDO A EXISTENCIA DE ERRO
        erro_pop_up = await is_window_open("Erro")
        if erro_pop_up["IsOpened"] == True:
            error_work = await error_after_xml_imported()
            return RpaRetornoProcessoDTO(
                sucesso=error_work.sucesso,
                retorno=error_work.retorno,
                status=error_work.status,
                tags=error_work.tags
            )
        
        max_attempts = 50
        i = 0
        while i < max_attempts:
            # VERIFICANDO A EXISTENCIA DE ERRO
            erro_pop_up = await is_window_open("Erro")
            if erro_pop_up["IsOpened"] == True:
                error_work = await error_after_xml_imported()
                return RpaRetornoProcessoDTO(
                    sucesso=error_work.sucesso,
                    retorno=error_work.retorno,
                    status=error_work.status,
                    tags=error_work.tags
                )
            information_pop_up = await is_window_open("Information")
            if information_pop_up["IsOpened"] == True:
                break
            else:
                console.print(f"Aguardando confirmação de nota incluida...\n")
                await worker_sleep(5)
                i += 1
        
        await worker_sleep(15)
        console.print("\nVerifica se a nota ja foi lançada...")
        nf_imported = await check_nota_importada(nota.get("nfe"))
        if nf_imported.sucesso == True:
            await worker_sleep(12)
            console.print("\nVerifica se a nota ja foi lançada...")
            status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
            if status_nf_emsys.get("status") == "Lançada":
                console.print("\nNota lançada com sucesso, processo finalizado...", style="bold green")
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno="Nota Lançada com sucesso!",
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
            else:
                console.print("Erro ao lançar nota", style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Pop-up nota incluida encontrada, porém nota encontrada como 'já lançada' trazendo as seguintes informações: {nf_imported.retorno} - {error_work}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
        else:
            console.print("Erro ao lançar nota", style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao lançar nota, erro: {nf_imported.retorno}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}



import os
import re
import shutil
import stat
import subprocess
import zipfile

import git
import toml
from pathlib3x import Path
from playwright.async_api import async_playwright
from rich.console import Console

from worker_automate_hub.config.settings import (
    get_package_version,
    load_env_config,
)
from worker_automate_hub.decorators.retry import retry
from worker_automate_hub.utils.logger import logger

console = Console()


def write_env_config(env_dict: dict, google_credentials: dict):
    try:
        current_dir = Path.cwd()
        assets_path = current_dir / "assets"
        logs_path = current_dir / "logs"
        assets_path.mkdir(exist_ok=True)
        logs_path.mkdir(exist_ok=True)

        config_file_path = current_dir / "settings.toml"
        config_data = {
            "name": "WORKER",
            "params": {
                "api_base_url": env_dict["API_BASE_URL"],
                "api_auth": env_dict["API_AUTHORIZATION"],
                "notify_alive_interval": env_dict["NOTIFY_ALIVE_INTERVAL"],
                "version": get_package_version("worker-automate-hub"),
                "log_level": env_dict["LOG_LEVEL"],
                "drive_url": env_dict["DRIVE_URL"],
                "xml_default_folder": env_dict["XML_DEFAULT_FOLDER"],
                "git_assets_url": env_dict["GIT_ASSETS_URL"],
            },
            "google_credentials": google_credentials["content"],
        }

        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)

        log_msg = f"Arquivo de configuração do ambiente criado em {config_file_path}"
        logger.info(log_msg)
        console.print(f"\n{log_msg}\n", style="green")

        return {
            "Message": log_msg,
            "Status": True,
        }
    except Exception as e:
        err_msg = f"Erro ao criar o arquivo de configuração do ambiente. Comando retornou: {e}"
        logger.error(err_msg)
        return {
            "Message": err_msg,
            "Status": False,
        }


def add_worker_config(worker):
    try:
        current_dir = Path.cwd()
        config_file_path = current_dir / "settings.toml"

        if not config_file_path.exists():
            raise FileNotFoundError(
                f"O arquivo de configuração não foi encontrado em: {config_file_path}"
            )

        with open(config_file_path, "r") as config_file:
            config_data = toml.load(config_file)

        config_data["id"] = {
            "worker_uuid": worker["uuidRobo"],
            "worker_name": worker["nomRobo"],
        }

        with open(config_file_path, "w") as config_file:
            toml.dump(config_data, config_file)

        log_msg = f"Informações do worker adicionadas ao arquivo de configuração em {config_file_path}"
        console.print(f"\n{log_msg}\n", style="green")
        return {
            "Message": log_msg,
            "Status": True,
        }
    except Exception as e:
        err_msg = f"Erro ao adicionar informações do worker ao arquivo de configuração.\n Comando retornou: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        return {
            "Message": err_msg,
            "Status": False,
        }


async def install_playwright():
    try:
        result1 = subprocess.run(
            ["pipx", "install", "playwright"],
            check=True,
            capture_output=True,
            text=True,
        )
        logger.info(result1.stdout)
        result2 = subprocess.run(
            ["playwright", "install"], check=True, capture_output=True, text=True
        )
        logger.info("Playwright instalado com sucesso!")
        logger.info(result2.stdout)
    except subprocess.CalledProcessError as e:
        logger.error(f"Erro ao instalar Playwright: {e}")


def extract_zip(zip_path, extract_to):
    """
    Extrai um arquivo ZIP para um diretório especificado.

    :param zip_path: Caminho para o arquivo ZIP.
    :param extract_to: Diretório onde os arquivos serão extraídos.
    """
    if not os.path.exists(extract_to):
        os.makedirs(extract_to)

    with zipfile.ZipFile(zip_path, "r") as zip_ref:
        zip_ref.extractall(extract_to)
        console.print(f"\nArquivos extraídos para {extract_to}\n", style="green")


def extract_drive_folder_id(url: str) -> str:
    """
    Extrai o ID da pasta do Google Drive a partir de uma URL.

    Args:
        url (str): URL da pasta no Google Drive.

    Returns:
        str: O ID da pasta ou uma exceção ValueError se o ID não for encontrado.
    """
    match = re.search(r"folders/([a-zA-Z0-9-_]+)", url)
    if match:
        return match.group(1)
    else:
        raise ValueError("ID da pasta não encontrado na URL.")


def handle_remove_readonly(func, path, exc_info):
    # Remove o atributo de somente leitura e tenta novamente
    os.chmod(path, stat.S_IWRITE)
    func(path)


def clone_public_repo(repo_url: str, local_path: str):
    try:
        console.print(f"Clonando o repositório {repo_url} para {local_path}...")
        git.Repo.clone_from(repo_url, local_path)
        console.print(
            f"Repositório clonado com sucesso em {local_path}.", style="green"
        )
    except Exception as e:
        console.print(f"Erro ao clonar o repositório: {e}", style="red")


@retry(max_retries=5, delay=10)
def update_assets_v2():
    """
    Atualiza os assets baixando os arquivos do repositório e substituindo os existentes.
    """
    console.print("\nAtualizando os assets...\n", style="bold blue")
    try:
        env_config, _ = load_env_config()
        current_dir = f"{Path.cwd()}\\assets"
        repo_url = env_config["GIT_ASSETS_URL"]

        if not os.path.exists(current_dir):
            # Faz o clone do repositório
            console.print(
                f"Clonando o repositório {repo_url} para {current_dir}...", style="blue"
            )
            clone_public_repo(repo_url, current_dir)

            console.print(
                "\nTodos os arquivos e pastas foram baixados e salvos na pasta 'assets' com sucesso.\n",
                style="bold green",
            )
        else:
            try:
                console.print(f"Atualizando o repositório em {current_dir}...")
                repo = git.Repo(current_dir)
                origin = repo.remotes.origin
                origin.fetch()
                origin.pull()
                console.print(
                    f"Repositório atualizado com sucesso em {current_dir}.",
                    style="green",
                )
            except:
                console.print(
                    f"Falha ao tentar atualizar, iniciando a limpeza...", style="red"
                )
                shutil.rmtree(current_dir, onerror=handle_remove_readonly)

                console.print(
                    f"Clonando o repositório {repo_url} para {current_dir}...",
                    style="blue",
                )
                clone_public_repo(repo_url, current_dir)

                console.print(
                    "\nTodos os arquivos e pastas foram baixados e salvos na pasta 'assets' com sucesso.\n",
                    style="bold green",
                )

    except Exception as e:
        console.print(f"\nErro ao atualizar os assets: {e}...\n", style="bold red")


async def install_tesseract(setup_path: Path):
    try:
        # Comando para executar com elevação
        command = f'start-process "{setup_path}" -Verb runAs'

        # Executar o comando usando PowerShell
        result = subprocess.run(
            ["powershell", "-Command", command], capture_output=True
        )
        logger.info(result.stdout)
        logger.info("Tesseract instalado com sucesso!")
    except subprocess.CalledProcessError as e:
        logger.error(f"Erro ao instalar Tesseract: {e}")


async def download_tesseract():
    # Diretório de execução atual
    current_dir = Path.cwd()
    output_folder = current_dir / "temp"
    destination_path = output_folder / "tesseract.exe"

    folder_url = "https://github.com/UB-Mannheim/tesseract/wiki"
    async with async_playwright() as p:
        browser = await p.chromium.launch(
            headless=True
        )  # Defina headless=True se não quiser ver o navegador
        context = await browser.new_context()

        # Abrir uma nova página
        page = await context.new_page()

        # Navegar para a URL do arquivo no Google Drive
        await page.goto(folder_url)

        # Aguarde o carregamento da página e clique no botão de download
        await page.wait_for_selector(
            "#wiki-body > div > ul:nth-child(7) > li > a",
            timeout=60000,
        )
        await page.click("#wiki-body > div > ul:nth-child(7) > li > a")

        # Aguarde o download começar
        download = await page.wait_for_event("download")

        # Salve o arquivo no destino especificado
        await download.save_as(destination_path)

        await browser.close()

    await install_tesseract(destination_path)



import asyncio
import getpass
from datetime import datetime

from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from rich.console import Console
import re
import os
from pywinauto.keyboard import send_keys
from worker_automate_hub.utils.util import login_emsys
import warnings
from pywinauto.application import Application
from worker_automate_hub.api.client import get_config_by_name, get_status_cte_emsys
from worker_automate_hub.api.ahead_service import save_xml_to_downloads
from worker_automate_hub.utils.util import (
    kill_all_emsys,
    delete_xml,
    set_variable,
    type_text_into_field,
    worker_sleep,
)
from pywinauto_recorder.player import set_combobox

from datetime import timedelta
import pyautogui
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.utils_nfe_entrada import EMSys
from datetime import datetime

ASSETS_BASE_PATH = "assets/cte_xml/"
emsys = EMSys()

console = Console()
pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False

data_hoje = datetime.now().strftime("%d%m%Y")
print(data_hoje)

async def janela_nota_fiscal(nota_entrada):
    # Declaração das variáveis
    data_emissao = nota_entrada["dataEmissao"]
    nota_fiscal = nota_entrada["numeroNota"]
    cnpj_fornecedor = nota_entrada["cnpjFornecedor"]
    try:
        await worker_sleep(5)
        app = Application(backend="win32").connect(
            title="Nota Fiscal de Entrada", found_index=0
        )
        main_window = app["Nota Fiscal de Entrada"]

        input_tipo_documento = main_window.child_window(class_name="TDBIComboBox", found_index=1)
    

        # Focar e clicar no ComboBox
        input_tipo_documento.set_focus()
        input_tipo_documento.click_input()
        
        await worker_sleep(1)

        # Selecionar Nota Fiscal de Entrada
        input_tipo_documento.select("NOTA FISCAL DE ENTRADA - SERVIÇO")
        
        await worker_sleep(3)
        
        # Enviar ENTER para confirmar
        send_keys("{ENTER}")
            
        await worker_sleep(2)
        
        # Inserir Entrada data_hoje
        input_entrada = main_window.child_window(class_name="TDBIEditDate", found_index=1)
        type_text_into_field(data_hoje, input_entrada, True, "10")
        
        # Inserir Data de Emissão
        input_emissao = main_window.child_window(class_name="TDBIEditDate", found_index=2)
        type_text_into_field(data_emissao, input_emissao, True, "10")
        
        # Inserir Nota Fiscal
        input_nota_fiscal = main_window.child_window(class_name="TDBIEditString", found_index=8)
        type_text_into_field(nota_fiscal, input_nota_fiscal, True, "50")
        
        # Inserir Serie
        input_serie = main_window.child_window(class_name="TDBIEditString", found_index=2)
        type_text_into_field("S", input_serie, True, "10")
        
        # Inserir CNPJ
        input_cnpj = main_window.child_window(class_name="TDBIEditString", found_index=4)
        type_text_into_field(cnpj_fornecedor, input_cnpj, True, "14")
    
        # Clicar na lupa
        pyautogui.click(x=980, y=504)
        
        # Inserir NOP Nota
        select_nop_nota = main_window.child_window(class_name="TDBIComboBox", found_index=0)
        
        # Focar e clicar no ComboBox
        select_nop_nota.set_focus()
        select_nop_nota.click_input()
        
        await worker_sleep(1)

        # Selecionar Opção
        select_nop_nota.select("1933 - AQUISIÇÃO DE SERVIÇOS - 1933")
        
        # Enviar ENTER para confirmar
        send_keys("{ENTER}")
        
        await worker_sleep(1)
        
        # Inserir Mod Frete
        select_mod_frete = main_window.child_window(class_name="TDBIComboBoxValues", found_index=1)
        
        # Focar e clicar no ComboBox
        select_mod_frete.set_focus()
        select_mod_frete.click_input()
        
        await worker_sleep(1)

        # Selecionar Opção
        select_mod_frete.select("9=Sem Ocorrência de Transporte.")
        
        # Enviar ENTER para confirmar
        send_keys("{ENTER}")   
         
        await worker_sleep(1)
        
        # Clicar no Itens da Nota
        pyautogui.click(x=628, y=328)
        
        await worker_sleep(3)
        
        # Clicar no sinal de +
        sinal_mais = main_window.child_window(class_name="TDBIBitBtn", found_index=3)
        
        # Focar e clicar no ComboBox
        sinal_mais.set_focus()
        sinal_mais.click_input()
    
        await worker_sleep(5)
            

    except Exception as e:
        # console.print(f"Erro ao conectar a janela {window_title}")
        raise Exception(f"Erro ao conectar a janela: {e}")

async def janela_inclusao_itens(nota_entrada):
    # Declaracao de variáveis
    codigo_empresa = nota_entrada["codigoEmpresa"]
    valor_nota = nota_entrada["valorNota"]
    conta_contabil = nota_entrada["contaContabil"]
    quantidade = nota_entrada["quantidade"]
    
    app = Application(backend="win32").connect(
        class_name="TFrmIncluiItemNFE", found_index=0
    )
    main_window = app["TFrmIncluiItemNFE"]

    # Inserir input_Almoxarifado
    input_almoxarifado = main_window.child_window(class_name="TDBIEditCode", found_index=2)
      
     # Focar e clicar no ComboBox
    input_almoxarifado.set_focus()
    
    
    if codigo_empresa != '1':
        codigo_empresa = f"{codigo_empresa}50"
    else:
        codigo_empresa = f"{codigo_empresa}60"
    
    type_text_into_field(codigo_empresa, input_almoxarifado, True, "10")
    
    send_keys("{ENTER}")
    
    await worker_sleep(3)
    
    # Inserir Item 
    input_item = main_window.child_window(class_name="TDBIEditNumber", found_index=40)
    type_text_into_field("2483", input_item, True, "10")
    send_keys("{TAB}")
    
    await worker_sleep(5)
    try:
        # Mudar para janela pesquisa itens
        app = Application(backend="win32").connect(
            class_name="TFrmPesquisaItem", found_index=0
        )
        main_window = app["TFrmPesquisaItem"]

        # Clicar Primeira LInha
        pyautogui.click(x=561, y=397)
        
        # Clicar em confirmar
        clicar_confirmar = main_window.child_window(class_name="TDBIBitBtn", found_index=2)
        clicar_confirmar.click_input()
    except:
        pass
    
    await worker_sleep(5)
    
    # Volta para janela de inclusão de itens
    app = Application(backend="win32").connect(
        class_name="TFrmIncluiItemNFE", found_index=0
    )
    main_window = app["TFrmIncluiItemNFE"]
    
    # Inserir quantidade
    input_quantidade = main_window.child_window(class_name="TDBIEditNumber", found_index=39)
    type_text_into_field(quantidade, input_quantidade, True, "10")
    
    # Inserir Valor Unitário
    input_valor_unitario = main_window.child_window(class_name="TDBIEditNumber", found_index=9)
      
     # Focar e clicar no ComboBox
    input_valor_unitario.set_focus()
    
    
    type_text_into_field(valor_nota, input_valor_unitario, True, "10")
    
    await worker_sleep(5)
    
    # Clicar em Tipo de Despesa
    pyautogui.click(x=793, y=485)
    
    await worker_sleep(2)
    
     # Inserir Tipo [Conta COntábil]
    input_tipo = main_window.child_window(class_name="TDBIEditCode", found_index=1)
      
     # Focar e clicar no ComboBox
    input_tipo.set_focus()
    
    
    type_text_into_field(conta_contabil, input_tipo, True, "10")
    
    # Clicar em Incluir 
    incluir = main_window.child_window(class_name="TDBIBitBtn", found_index=1)
    incluir.click_input()
    
    await worker_sleep(5)
    
    # Clicar em cancelar
    cancelar = main_window.child_window(class_name="TDBIBitBtn", found_index=2)
    cancelar.click_input()

async def janela_pagamento(nota_entrada):
    # Declarar variáveis
    data_vencimento = datetime.now() + timedelta(days=1)
    data_vencimento_formatada = data_vencimento.strftime('%d%m%Y')
    valor_nota = nota_entrada["valorNota"]
    # Clicar em pagamento
    pyautogui.click(x=623, y=360)
    
    await worker_sleep(2)
    
    app = Application(backend="win32").connect(
        class_name="TFrmNotaFiscalEntrada", found_index=0
    )
    main_window = app["TFrmNotaFiscalEntrada"]
    
    # Selecionar tipo de cobrança
    select_tipo_cobranca = main_window.child_window(class_name="TDBIComboBox", found_index=0)
    select_tipo_cobranca.select("BANCO DO BRASIL BOLETO")
    
    await worker_sleep(1)
    
    # Inserir data vencimento
    input_data_vencimento = main_window.child_window(class_name="TDBIEditDate", found_index=0)
    type_text_into_field(data_vencimento_formatada, input_data_vencimento, True, "10")
    
    await worker_sleep(1)
    
    # Inserir valor da nota
    input_valor_nota = main_window.child_window(class_name="TDBIEditNumber", found_index=3)
    type_text_into_field(valor_nota, input_valor_nota, True, "10")
    
    await worker_sleep(1)
    
    # Clicar no botão +
    clicar_mais = main_window.child_window(class_name="TDBIBitBtn", found_index=1)
    clicar_mais.click_input()
    
    await worker_sleep(2)
    
    # Clicar no + para salvar
    pyautogui.click(x=594, y=281)
    
    await worker_sleep(3)
    
    # Clicar em selecionar todos
    pyautogui.click(x=746, y=478)


async def janela_rateio_despesa(nota_entrada):
    await worker_sleep(5)
    # Declarar variável
    app = Application(backend="win32").connect(
        class_name="TFrmDadosRateioDespesa", found_index=0
    )
    main_window = app["TFrmDadosRateioDespesa"]   

    for rateio in nota_entrada["rateios"]:
        centro_custo = rateio["centroCusto"]
        valor = rateio["valor"]
        print(f"Centro de Custo: {centro_custo} | Valor: {valor}") 
        
        # Inserir centro de custo
        input_centro_custo = main_window.child_window(class_name="TDBIEditCode", found_index=0)
        type_text_into_field(centro_custo, input_centro_custo, True, "10")
        send_keys("{TAB}")
        
        await worker_sleep(1)
       
        # Inserir valor
        input_valor = main_window.child_window(class_name="TDBIEditNumber", found_index=5)
        type_text_into_field(valor, input_valor, False, "10")
        
        await worker_sleep(1)
        
        # Clicar em incluir rateio
        clicar_incluir_rateio = main_window.child_window(class_name="TDBIBitBtn", found_index=3)
        clicar_incluir_rateio.click()
        
        await worker_sleep(2)

async def lancamento_rateio(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    try:
        config = await get_config_by_name("login_emsys")
        nota_entrada = task.configEntrada

        await kill_all_emsys()

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3_35.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field(
                "Nota Fiscal de Entrada", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )

            pyautogui.press("enter")
            await worker_sleep(2)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(6)

        await janela_nota_fiscal(nota_entrada)
        
        await janela_inclusao_itens(nota_entrada)
        
        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno=f"Suceso no processo Lançamento de Rateio",
            status=RpaHistoricoStatusEnum.Sucesso,
        )

    except Exception as e:
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro no processo CTE com XML: {e}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )
    
        

if __name__ == "__main__":
    task = RpaProcessoEntradaDTO(
        datEntradaFila=datetime.now(),
        configEntrada={
            "rateios": [
                {"valor": "3040,10", "centroCusto": "1133"},
                {"valor": "999,50", "centroCusto": "1155"},
                {"valor": "1516,53", "centroCusto": "1210"},
                {"valor": "4162,28", "centroCusto": "1154"},
                {"valor": "4145,93", "centroCusto": "1146"},
                {"valor": "3553,20", "centroCusto": "1136"},
                {"valor": "205,58", "centroCusto": "1071"},
                {"valor": "2270,66", "centroCusto": "1156"},
                {"valor": "2346,79", "centroCusto": "1088"},
                {"valor": "13720,75", "centroCusto": "1223"},
                {"valor": "6747,75", "centroCusto": "1035"},
                {"valor": "2886,55", "centroCusto": "1129"},
                {"valor": "3011,67", "centroCusto": "1130"},
                {"valor": "2103,99", "centroCusto": "1054"},
                {"valor": "2544,05", "centroCusto": "1077"},
                {"valor": "3308,69", "centroCusto": "1091"},
                {"valor": "3104,85", "centroCusto": "1093"},
                {"valor": "1507,34", "centroCusto": "1092"},
                {"valor": "4143,30", "centroCusto": "1103"},
                {"valor": "4774,93", "centroCusto": "1060"},
                {"valor": "1743,58", "centroCusto": "9999"},
                {"valor": "3783,73", "centroCusto": "1053"},
                {"valor": "3141,57", "centroCusto": "1041"},
                {"valor": "2586,06", "centroCusto": "1099"},
                {"valor": "2286,40", "centroCusto": "1166"},
                {"valor": "2848,49", "centroCusto": "1119"},
                {"valor": "3428,97", "centroCusto": "1078"},
                {"valor": "1672,26", "centroCusto": "1139"},
                {"valor": "1872,63", "centroCusto": "1124"},
                {"valor": "2060,27", "centroCusto": "1039"},
                {"valor": "5148,94", "centroCusto": "1062"},
                {"valor": "3182,39", "centroCusto": "1211"}
            ],
            "valorNota": "10849,75",
            "identificador": "aquiSeuIdentificador",
            "numeroNota": "127",
            "urlRetorno": "https://suaempresa.com.br/retorno-nota",
            "codigoEmpresa": "1",
            "nomeEmpresa": "MATRIZ",
            "contaContabil": "13",
            "cnpjFornecedor": "31492377000139",
            "dataEmissao": "23/05/2025",
            "quantidade": "1"
        },  # vírgula aqui é essencial
        uuidProcesso='b47f25e8-0b41-429d-904b-7db7a03219cc',
        nomProcesso='lancamento_rateio',
        uuidFila="",
        sistemas=[
            {"sistema": "EMSys", "timeout": "1.0"},
            {"sistema": "AutoSystem", "timeout": "1.0"}
        ],
        historico_id='01'
    )

    asyncio.run(lancamento_rateio(task))



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, Field

from worker_automate_hub.models.dao.rpa_robo import RpaRobo


class RpaFila(BaseModel):
    uuidFila: Optional[str] = Field(None, alias="uuidFila")
    robos: Optional[List[RpaRobo]] = Field(None, alias="robos")
    fkRobos: Optional[str] = Field(None, alias="fkRobos")
    uuidProcesso: str = Field(..., alias="uuidProcesso")
    prioridade: int = Field(..., alias="prioridade")
    configEntrada: Optional[dict] = Field(None, alias="configEntrada")
    dtLeituraFila: Optional[datetime] = Field(None, alias="dtLeituraFila")
    lock: Optional[bool] = Field(None, alias="lock")
    mutarNotificacao: Optional[int] = Field(None, alias="mutarNotificacao")

    class Config:
        populate_by_name = True



import asyncio
import warnings

import pyautogui
from rich.console import Console

from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import RpaProcessoEntradaDTO
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    find_element_center,
    find_target_position,
    kill_all_emsys,
    take_screenshot,
    type_text_into_field,
    wait_element_ready_win,
)

console = Console()


async def login_emsys(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    from pywinauto.application import Application

    warnings.filterwarnings(
        "ignore",
        category=UserWarning,
        message="32-bit application should be automated using 32-bit Python",
    )

    try:
        # Fecha a instancia do emsys - caso esteja aberta
        await kill_all_emsys()

        # Abre um novo emsys
        app = Application().start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        console.print("\nEMSys iniciando...", style="bold green")

        await asyncio.sleep(10)
        # Testa se existe alguma mensagem no Emsys
        window_message_login_emsys = await find_element_center(
            "assets/emsys/window_message_login_emsys.png", (560, 487, 1121, 746), 10
        )

        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()

        pyautogui.click(screen_width / 2, screen_height / 2)

        # Clica no "Não mostrar novamente" se existir
        if window_message_login_emsys:
            pyautogui.click(window_message_login_emsys.x, window_message_login_emsys.y)
            pyautogui.click(
                window_message_login_emsys.x + 383, window_message_login_emsys.y + 29
            )
            console.print("Mensagem de login encontrada e fechada.", style="bold green")

        # Ve se o Emsys esta aberto no login
        image_emsys_login = await find_element_center(
            "assets/emsys/logo_emsys_login.png", (800, 200, 1400, 700), 600
        )
        # config_robot = await get_config_by_name("Login EmSys")
        if image_emsys_login:
            # await asyncio.sleep(10)
            # type_text_into_field(
            #     config_robot["EmSys DB"], app["Login"]["ComboBox"], True, "50"
            # )
            # pyautogui.press("enter")
            # await asyncio.sleep(2)

            if await wait_element_ready_win(app["Login"]["Edit2"], 30):
                disconect_database = await find_element_center(
                    "assets/emsys/disconect_database.png", (1123, 452, 1400, 578), 300
                )

                if disconect_database:
                    # Realiza login no Emsys
                    type_text_into_field(
                        task.configEntrada["user"], app["Login"]["Edit2"], True, "50"
                    )
                    pyautogui.press("tab")
                    type_text_into_field(
                        task.configEntrada["pass"],
                        app["Login"]["Edit1"],
                        True,
                        "50",
                    )
                    pyautogui.press("enter")

                    # Seleciona a filial do emsys
                    selecao_filial = await find_element_center(
                        "assets/emsys/selecao_filial.png", (480, 590, 820, 740), 15
                    )

                    if selecao_filial == None:
                        screenshot_path = take_screenshot()
                        selecao_filial = find_target_position(
                            screenshot_path, "Grupo", 0, -50, 15
                        )

                        if selecao_filial == None:
                            selecao_filial = (804, 640)

                        pyautogui.write(task.configEntrada["filial"])
                        pyautogui.press("enter")

                    else:
                        type_text_into_field(
                            task.configEntrada["filial"],
                            app["Seleção de Empresas"]["Edit"],
                            True,
                            "50",
                        )
                        pyautogui.press("enter")

                    button_logout = await find_element_center(
                        "assets/emsys/button_logout.png", (0, 0, 130, 150), 75
                    )

                    if button_logout:
                        console.print(
                            "Login realizado com sucesso.", style="bold green"
                        )
                        return RpaRetornoProcessoDTO(
                            sucesso=True,
                            retorno="Processo de login no EMSys executado com sucesso.",
                            status=RpaHistoricoStatusEnum.Sucesso,
                        )
                       
            else:
                logger.info("login_emsys_win -> wait_element_ready_win [1]")
                console.print("Elemento de login não está pronto.", style="bold red")

    except Exception as ex:
        logger.error("Erro em login_emsys: " + str(ex))
        console.print(f"Erro em login_emsys: {str(ex)}", style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro em login_emsys: {str(ex)}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )
        



import functools
import time


def rate_limit(calls_per_period: int, period: float) -> callable:
    """
    Um decorador que limita o número de vezes que uma função pode ser chamada em um determinado período.

    O decorador recebe dois argumentos:
    - calls_per_period: O número máximo de vezes que a função pode ser chamada em um determinado período.
    - period: O período durante o qual a função pode ser chamada.

    O decorador retorna uma função que encapsula a função original.

    A função encapsulada manterá o controle dos timestamps das últimas chamadas calls_per_period.
    Se o número de timestamps exceder calls_per_period, uma exceção será gerada.

    :param calls_per_period: O número máximo de vezes que a função pode ser chamada em um determinado período.
    :param period: O período durante o qual a função pode ser chamada.
    :return: A função decorada.
    """

    def decorator(func):
        timestamps = []

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            nonlocal timestamps
            current_time = time.time()
            timestamps = [t for t in timestamps if current_time - t < period]
            if len(timestamps) >= calls_per_period:
                raise Exception("Limite de chamadas excedido")
            timestamps.append(current_time)
            return func(*args, **kwargs)

        return wrapper

    return decorator



import asyncio
import io
import os
import re
import shutil
import warnings
from datetime import datetime, timedelta


import pyperclip
import pyautogui
from pywinauto.application import Application
from rich.console import Console
from dateutil.relativedelta import relativedelta
from pywinauto import Desktop

from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import RpaProcessoEntradaDTO
from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
import win32clipboard
from googleapiclient.discovery import build
from worker_automate_hub.utils.logger import logger
from pywinauto_recorder import set_combobox
from worker_automate_hub.api.client import get_config_by_name, get_valor_remessa_cobranca, send_file, sync_get_config_by_name
from worker_automate_hub.utils.util import (
    capture_and_send_screenshot,
    create_temp_folder,
    delete_folder,
    kill_all_emsys,
    login_emsys,
    save_pdf_emsys, 
    type_text_into_field,
    worker_sleep,
    set_variable,
    )

pyautogui.PAUSE = 0.5
ASSETS_BASE_PATH = 'assets/fidc/'
PDF_SUFFIX = "_PDF.pdf"
console = Console()



async def exportacao_docs_portal_b2b(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    '''
       Processo FIDC - Remessa de Cobrança CNAB240
    '''
    try:
        #Setando tempo de timeout
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)
        
        #Pegando nome do usuario
        nome_usuario = os.environ.get('USERNAME') or os.environ.get('USER')
        nome_pasta = f"{nome_usuario}_arquivos"

        #Delete temp folder
        await delete_folder(nome_pasta)
        #Cria Pasta temporaria
        temp_folder = await create_temp_folder()
        temp_folder = temp_folder +'\\'
        #Pega Config para logar no Emsys
        config = await get_config_by_name("login_emsys")
        folders_paths = await get_config_by_name("Folders_Fidc")
        #Abre um novo emsys
        await kill_all_emsys()
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field('Envia Boleto por email', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            await worker_sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Envia Boleto por email' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login
        await worker_sleep(5)
        # Identificando jenela principal
        app = Application().connect(class_name="TFrmEnviaEmailBoletaTitulo", backend="win32")
        main_window_envia_boletos = app["TFrmEnviaEmailBoletaTitulo"]
        main_window_envia_boletos.set_focus()

        if task.uuidProcesso != "5ad2d209-e9da-438c-ba62-db0a5f9a3795":
            #Seleciona "Sim" na fatura
            main_window_envia_boletos.child_window(class_name="TRadioButton", title="Sim").click()

            #Seleciona tipo de cobrança
            main_window_envia_boletos.child_window(class_name="TDBIEditCode", found_index=1).type_keys("4044{TAB}")

            #Selecionando Modelo
            select_model = main_window_envia_boletos.child_window(class_name="TDBIComboBox", found_index=0)
            select_model.select("BANCO DO BRASIL FIDC")
        else:
            #Seleciona "Sim" na fatura
            main_window_envia_boletos.child_window(class_name="TRadioButton", title="Sim/Negociados").click()

            #Seleciona tipo de cobrança
            main_window_envia_boletos.child_window(class_name="TDBIEditCode", found_index=1).type_keys("1{TAB}")

            #Selecionando Modelo
            select_model = main_window_envia_boletos.child_window(class_name="TDBIComboBox", found_index=0)
            select_model.select("BANCO DO BRASIL BOLETO")

        #Desmarcando Frete/Carregamento
        main_window_envia_boletos.child_window(class_name="TDBICheckBox", found_index=7).click()
        #Desmarcando PDF do Ct-e
        main_window_envia_boletos.child_window(class_name="TDBICheckBox", found_index=4).click()
        #Selecionando "Somente Não Enviados"
        main_window_envia_boletos.child_window(class_name="TDBICheckBox", found_index=6).click()
        #Selecionando "Enviar XML e PDF do NF-e"
        main_window_envia_boletos.child_window(class_name="TDBICheckBox", found_index=3).click()
        #Selecionando "Cupom Fiscal"
        main_window_envia_boletos.child_window(class_name="TDBICheckBox", found_index=13).click()

        #Selecionando "Enviar Resumo da Fatura"
        main_window_envia_boletos.child_window(class_name="TDBICheckBox", found_index=5).click()
        
        await worker_sleep(6)
        try:
            app = Application().connect(class_name="TMessageForm", backend="uia", title="Confirm")
            window_template = app["TMessageForm"]
            window_template.set_focus()
            await worker_sleep(2)
            #Clica em "No" no aviso
            window_template.child_window(title="&No", class_name="TButton").click()
            pyautogui.click(1002, 561)
            await worker_sleep(2)
            #Clica em Detalhamento da Fatura
            pyautogui.click(916, 457)
            await worker_sleep(2)
            #Clica em Ok na janelinha de detalhamento
            pyautogui.click(1046, 556)
            await worker_sleep(2)
        except Exception as ex:
            log_msg = f"Erro ao identificar a janela Resumo da fatura: {str(ex)}"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        

        #Digita Periodo de Emissão
        # Data atual
        data_atual = datetime.now()
        # Data(8 dias atrás)
        start_date = data_atual - timedelta(days=8)

        # Data(1 dia atrás)
        end_date = data_atual - timedelta(days=1)
        main_window_envia_boletos.child_window(class_name="TDBIEditDate", found_index=1).type_keys(f"{start_date:%d/%m/%Y}")

        await worker_sleep(1)

        main_window_envia_boletos.child_window(class_name="TDBIEditDate", found_index=0).type_keys(f"{end_date:%d/%m/%Y}")

        await worker_sleep(2)   

        #Clica em Pesquisar
        pyautogui.click(602, 588)
        await worker_sleep(5)

        #Deseja selecionar as empresas para pesquisa?
        try:
            app = Application().connect(class_name="TMessageForm", backend="uia", title="Confirm")
            window_company_select = app["TMessageForm"]
            window_company_select.set_focus()
            window_company_select.child_window(title="&Yes", class_name="TButton").click()
            await worker_sleep(5)

            app = Application().connect(class_name="TFrmSelecionaEmpresas", backend="uia", title="Seleção de Empresas")
            window_company_select = app["TFrmSelecionaEmpresas"]
            if window_company_select.exists():
                window_company_select.set_focus()
                pyautogui.click(720, 623)
                await worker_sleep(5)
                #Remove filial 189
                window_company_select.child_window(class_name='TEdit').type_keys('189')
                await worker_sleep(2)
                pyautogui.click(690, 440)
        except Exception as ex:
            log_msg = f"Erro ao identificar a janela de empresas: {str(ex)}"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        
        pyautogui.click(1092, 658)
        await worker_sleep(2)
        #Aguarda botão pesqiusar inativar
        current_try = 0
        max_try = 10
        while current_try < max_try:
            try:
                pyautogui.locateOnScreen(ASSETS_BASE_PATH + 'envio_docs_b2b_pesquisar_desabilitado.png', confidence=0.9)
                break
            except Exception as e:
                await worker_sleep(20)
                current_try += 1
                continue
        
        if current_try >= max_try:
            log_msg = f"Numero máximo de tentativas para verificar o botão pesquisar desabilitado excedidas"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])

        #Desmarca titulos sem Nosso Número gerado
        
        #Clica para selecionar todos registros
        pyautogui.click(970,580)

        #Filtra registros para aparecerem em ordem
        pyautogui.click(700,680)
        await worker_sleep(2)

        pyautogui.click(615, 700)
        #Vai para o primeiro item
        with pyautogui.hold('ctrl'):
            pyautogui.hotkey('home')
        pyautogui.hotkey('home')

        pyautogui.PAUSE = 0.3
        while True:
            try:
                console.print("[green]Copiando Linha")
                with pyautogui.hold('ctrl'):
                    pyautogui.press('c')

                win32clipboard.OpenClipboard()
                actual_line = win32clipboard.GetClipboardData()
                win32clipboard.CloseClipboard()

                if actual_line.split('\n')[1].split('\t')[2] == '':
                    console.print("[yellow]Desmarcando item para envio")
                    pyautogui.press('space')
                else:
                    console.print("[red]Encerrando o Loop")
                    break

                console.print("[yellow]Seguindo para próxima linha")
                pyautogui.press('down')

            except Exception as ex:
                log_msg = f"Erro ao marcar item: {str(ex)}"
                console.print(log_msg, style="bold red")
        
        #Envaindo emails
        pyautogui.click(700, 585)

        #Deseja enviar boletos por email?
        await worker_sleep(15)
        try:
            app = Application().connect(class_name="TMessageForm", backend="uia", title="Information")
            window_boleto_email = app["TMessageForm"]
            window_boleto_email.set_focus()
            window_boleto_email.child_window(title="&Yes", class_name="TButton").click()
            #pyautogui.click(915, 560)
            await worker_sleep(12)
        except Exception as ex:
            log_msg = f"Erro ao achar janela de confirmação de envio de emails: {str(ex)}"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        
        #Deseja marcar boletos enviados como impressos
        try: 
            app = Application().connect(class_name="TMessageForm", backend="uia", title="Confirm")
            window_boleto_email = app["TMessageForm"]
            window_boleto_email.set_focus()
            window_boleto_email.child_window(title="&Yes", class_name="TButton").click()
            pyautogui.click(915, 560)
            await worker_sleep(12)
        except Exception as ex:
            log_msg = f"Erro ao achar janela de confirmação de envio de emails: {str(ex)}"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        
        await worker_sleep(15)
        sucesso = False
        while not sucesso:
            desktop = Desktop(backend="uia")
            try:
                # Tenta localizar a janela com o título que contém "Aguarde"
                window = desktop.window(title_re="Aguarde...")
                # Se a janela existe, continua monitorando
                if window.exists():
                    console.print(f"Janela 'Aguarde...' ainda aberta", style="bold yellow")
                    logger.info(f"Janela 'Aguarde...' ainda aberta")
                    await worker_sleep(30)
                    continue
                else:
                    try:
                        desktop_second = Desktop(backend="uia")
                        window_aguarde = desktop_second.window(title_re="Aguarde...")
                        if not window_aguarde.exists():
                            sucesso = True
                            break
                        else:
                            console.print(f"Janela 'Aguarde...' ainda aberta. Seguindo para próxima tentativa", style="bold yellow")
                            continue
                    except:
                        console.print(f"Janela 'Aguarde...' não existe mais.", style="bold green")
                        await worker_sleep(5)
                        break
                
            except Exception as ex:
                log_msg= f"Erro ao verificar janela 'Aguarde...': {str(ex)}"
                console.print(log_msg, style="bold red")
                return RpaRetornoProcessoDTO(sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])

        if sucesso:
            await capture_and_send_screenshot(task.historico_id, "PortalB2B_Exportacao_Documentos")
            return RpaRetornoProcessoDTO(sucesso=True, retorno="Processo de exportação dos documentos do portal B2B finalizado", status=RpaHistoricoStatusEnum.Sucesso)
        else:
            return RpaRetornoProcessoDTO(sucesso=False, retorno="Ocorreu um erro ao finalizar o processo de exportação dos documentos do portal B2B", status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
    except Exception as ex:
        log_msg = f"Erro Processo Exportacao Docs Portal B2B: {str(ex)}"
        console.print(log_msg, style="bold red")
        return RpaRetornoProcessoDTO(
        sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])


from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import RpaProcessoEntradaDTO


async def cte_manual(task: RpaProcessoEntradaDTO):
    chaveCte = task.configEntrada.get("chaveCte")
    return RpaRetornoProcessoDTO(
        sucesso=False,
        retorno=f"CTE Não automatizado. Realize a importação manualmente. CHAVE: {chaveCte}",
        status=RpaHistoricoStatusEnum.Falha,
        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
    )



import getpass
import io
import os
import re
from pathlib import Path
from typing import List, Tuple

import pyautogui
from googleapiclient.discovery import build
from googleapiclient.http import MediaIoBaseDownload
from pywinauto.application import Application
from pywinauto.keyboard import send_keys
from pywinauto_recorder.player import set_combobox
from rich.console import Console
from datetime import datetime, timedelta

from worker_automate_hub.api.client import get_status_cte_emsys, get_status_nf_emsys
from worker_automate_hub.models.dao.rpa_configuracao import RpaConfiguracao
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import worker_sleep, is_window_open_by_class

pyautogui.PAUSE = 0.5
import pyperclip

ASSETS_PATH = "assets"

console = Console()


class EMSys:
    def __init__(self):
        self.scrolled_pixels = 0
        self.edited_sequences = set()
        self.apenas_isqueiros = False

    async def click_itens_da_nota(self):
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]

        main_window.set_focus()
        console.print("Acessando os itens da nota... \n")
        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TTabSheet = panel_TPage.child_window(class_name="TcxCustomInnerTreeView")
        panel_TTabSheet.wait("visible")
        panel_TTabSheet.click()
        send_keys("{DOWN " + ("5") + "}")
        if await self.is_itens_da_nota_accessed():
            console.print(f"Itens da nota acessados com sucesso...\n")
        # Caso False raise Exception com a mensagem?
        await worker_sleep(2)

    async def is_itens_da_nota_accessed(self):
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]
        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TPage.wait("visible")
        panel_TTabSheet = panel_TPage.child_window(class_name="TTabSheet")
        title_n_serie = panel_TPage.child_window(title="N° Série")

        console.print("Verificando se os itens foram abertos com sucesso... \n")
        if not title_n_serie:
            console.print(f"Não foi possivel acessar a aba de 'Itens da nota...\n")
            # poderia ser return False ?
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Não foi possivel acessar a aba de 'Itens da nota'",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        else:
            return True

    async def alterar_item(self):
        try:
            pyautogui.click(745, 486)
            await worker_sleep(5)
            set_combobox("||List", "UNIDADE")

            # Check Alterar Trib. Manual
            pyautogui.click(1177, 740)
            # pyautogui.moveTo(1177, 740)

            # Click Alterar
            await worker_sleep(1)
            pyautogui.click(1180, 776)
            await worker_sleep(1)

            await worker_sleep(1)
            await self.verify_warning_and_error("Confirm")
            await worker_sleep(1)
        except Exception as err:
            console.print(f"Não foi possível alterar o item: {err}", style="bold red")

    async def verify_warning_and_error(self, warning_title, title="&Yes"):
        try:
            console.print("Verificando a existencia de warnings e errors\n")
            app = Application().connect(title=warning_title)
            main_window = app[warning_title]
            main_window.set_focus()

            console.print(f"Clicando em {title}, para andamento do processo...\n")
            btn_no = main_window.child_window(title=title)
            if btn_no.exists(timeout=5) and btn_no.is_enabled():
                btn_no.set_focus()
                btn_no.click()
                await worker_sleep(3)
            else:
                console.print(
                    "Warning - Erro durante a verificação de warnings e errors"
                )
                return {
                    "sucesso": False,
                    "retorno": "Erro durante a verificação de warnings e errors\n",
                }

        except:
            console.print("Não possui nenhum warning após a importação da nota...\n")

    async def percorrer_grid(self, cnpj=None):
        self.apenas_isqueiros = True
        await self.click_itens_da_nota()
    

        console.print("Acessando os itens indivualmente... \n")
        send_keys("{TAB 2}", pause=0.1)
        await worker_sleep(2)
        index_item_atual = 0
        index_ultimo_item = await self.get_ultimo_item()
        console.print(f"Index ultimo item: {index_ultimo_item}")
        
        try:
            while index_item_atual < index_ultimo_item:
                send_keys("^({HOME})")
                await worker_sleep(1)

                if index_item_atual > 0:
                    send_keys("{DOWN " + str(index_item_atual) + "}")

                await worker_sleep(2)
                send_keys("+{F10}")
                await worker_sleep(1)
                send_keys("{DOWN 2}")
                await worker_sleep(1)
                send_keys("{ENTER}")

                await worker_sleep(2)
                app = Application().connect(title="Alteração de Item")
                main_window = app["Alteração de Item"]

                main_window.set_focus()

                edit = main_window.child_window(
                    class_name="TDBIEditCode", found_index=0
                )
                
                #seta IPI 0%
                ipi_combobox = main_window.child_window(
                    class_name="TDBIComboBox", found_index=4
                )
                ipi_combobox.select("IPI 0%")
                console.print(f"edit text: {edit.window_text()}")

                # ITERANDO COM O ITEM
                tpage_item = main_window.child_window(
                    class_name="TPanel", found_index=0
                )
                item = tpage_item.child_window(
                    class_name="TDBIEditString", found_index=1
                )
                item_value = item.window_text()

                console.log(f"item name: {item_value}")

                # Verifica se o item é 'isqueiro'
                if ("isqueiro" in item_value.lower() or "acendedor" in item_value.lower()) and cnpj != "36359969000109":

                    try:
                        console.print(f"Trabalhando com os itens, alterando o código de tributação dos ISQUEIROS/ACENDEDORES para manual \n")
                        console.print("Item encontrado: ISQUEIRO/ACENDEDORES. Processando...\n")
                        
                        tipos_unidade = ["UNIDADE", "UN"]
                        for tipo in tipos_unidade:
                            try:     
                                await worker_sleep(4)
                                # Selecionar natureza da operação 9 - outros
                                select_other = main_window.child_window(
                                    class_name="TDBIComboBox", found_index=1
                                ).click()
                                select_other.select(tipo)
                                # Check Alterar Trib. Manual
                                pyautogui.click(1177, 740)
                                # pyautogui.moveTo(1177, 740)
                                
                                # Click Alterar
                                await worker_sleep(1)
                                pyautogui.click(1180, 776)
                                await worker_sleep(1)

                                await worker_sleep(1)
                                await self.verify_warning_and_error("Confirm")
                                await worker_sleep(1)
                            except Exception as e:
                                print(f"Não foi possível selecionar '{tipo}': {e}")

                        # await self.alterar_item()
                        await worker_sleep(3)
                        await self.click_itens_da_nota()
                    except Exception as error:
                        console.print(f"Erro ao alterar item: {error}")

                else:
                    self.apenas_isqueiros = False
                    console.print(
                        "Item não é isqueiro. Continuando para o próximo...\n"
                    )
                    # Caminho da imagem
                    caminho_imagem = "assets\\ipi_0.png"
                    # Verifica se o arquivo existe
                    if os.path.exists(caminho_imagem):
                        print("Imagem já existe. Clicando em 'Cancelar'...")
                                            
                        # Conectando à janela do app 
                        app = Application().connect(class_name="TFrmAlteraItemNFE", timeout=60)
                        janela_cancelar = app["TFrmAlteraItemNFE"]

                        # Encontra o botão pelo texto
                        try:
                            cancelar_btn = janela_cancelar.child_window(class_name ="TDBIBitBtn", found_index=2)
                            cancelar_btn.click_input()
                            print("Clicou em 'Cancelar'.")
                        except Exception as e:
                            print(f"Erro ao clicar no botão: {e}")
                    else:    
                        #Seleciona o IPI 0%
                        ipi_combobox = main_window.child_window(
                            class_name="TDBIComboBox", found_index=4
                        )
                        ipi_combobox.select("IPI 0%")

                        # Click Alterar
                        await worker_sleep(1)
                        pyautogui.click(1180, 776)
                        await worker_sleep(1)
                        await self.click_itens_da_nota()

                    # Fecha a janela de alteração de item
                    # try:
                    #     pyautogui.hotkey('esc')
                    #     console.print("Apertando [ESC]", style="bold blue")
                    # except Exception as e:
                    #     console.print(f"Erro ao cancelar edição de item: {e}", style="bold red")
                    # await worker_sleep(3)

                index_item_atual += 1
                console.print(f"Item aual no final da execução: {index_item_atual}")
                await worker_sleep(1)
                pyautogui.click(1083, 682)


        except Exception as e:
            return {
                "sucesso": False,
                "retorno": f"Erro aotrabalhar nas alterações dos itens: {e}",
            }

    async def click_principal(self):
        pyautogui.click(621, 327)
        await worker_sleep(2)

    async def select_tipo_cobranca(self):
        await worker_sleep(3)
        # pyautogui.click(632, 384)
        
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]

        main_window.set_focus()
        console.print("Acessando a aba de Pagamentos... \n")
        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TTabSheet = panel_TPage.child_window(class_name="TcxCustomInnerTreeView")
        try:
            panel_TTabSheet.wait("visible")
        except:
            await worker_sleep(2)
        panel_TTabSheet.click()
        send_keys("{DOWN " + ("2") + "}")
        
        await worker_sleep(2)
        # pyautogui.click(893, 549)
        await worker_sleep(5)
        try:
            caminho_imagem = "assets\\banco_boleto.png"
            # Verifica se apareceu a imagem de "sem dados"
            localizacao = pyautogui.locateOnScreen(caminho_imagem, confidence=0.9)
            if localizacao:
                pass
        
        except:
            try:   
                # Selecionar natureza da operação 9 - outros
                select_other = main_window.child_window(
                    class_name="TDBIComboBox", found_index=0
                )
                select_other.select("BOLETO")
            except Exception as e:
                print(f"Não foi possível selecionar: {e}")
        # try:
        #     set_combobox("||List", "BANCO DO BRASIL BOLETO")
        # except:
        #     set_combobox("||List", "BOLETO")

    async def inserir_vencimento_e_valor(self, nome_fornecedor, data_emissao, data_vencimento, valor):

        if float(valor) <= 0:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"O valor da nota não pode ser {valor}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
            )

        emission_date = datetime.strptime(data_emissao, '%d/%m/%Y')
        due_date = datetime.strptime(data_vencimento, '%d/%m/%Y')
        min_due_date = emission_date + timedelta(days=14)

        pyautogui.click(1030, 648)
        send_keys('^c')
        await worker_sleep(1)
        current_content = pyperclip.paste().strip()
        date_match = re.search(r'\d{2}/\d{2}/\d{4}', current_content)

        if 'SOUZA CRUZ' in nome_fornecedor.upper():
            if due_date < min_due_date and date_match is not None:
                new_due_date_str = min_due_date.strftime('%d/%m/%Y')
                pyautogui.click(891, 596)
                await worker_sleep(1)
                pyautogui.write(new_due_date_str)
                await worker_sleep(1)
                pyautogui.click(881, 620)
                await worker_sleep(1)
                pyautogui.press("delete")
                await worker_sleep(1)
                pyautogui.write(valor)
                await worker_sleep(1)
            elif date_match is None:
                new_due_date_str = min_due_date.strftime('%d/%m/%Y')
                pyautogui.click(891, 596)
                await worker_sleep(1)
                pyautogui.write(new_due_date_str)
                await worker_sleep(1)
                pyautogui.click(881, 620)
                await worker_sleep(1)
                pyautogui.press("delete")
                await worker_sleep(1)
                pyautogui.write(valor)
                await worker_sleep(1)
                pyautogui.click(1287, 547)

        elif date_match is None:
            pyautogui.click(891, 596)
            await worker_sleep(1)
            pyautogui.write(data_vencimento)
            await worker_sleep(1)
            pyautogui.click(881, 620)
            await worker_sleep(1)
            pyautogui.press("delete")
            await worker_sleep(1)
            pyautogui.write(valor)
            await worker_sleep(1)
            pyautogui.click(1287, 547)

        await worker_sleep(5)
        try:
            console.print("Verificando a existencia de warning acusando que a soma de pagamentos não bate com o valor da nota\n")
            app = Application().connect(title="Warning")
            main_window = app["Warning"]
            main_window.set_focus()

            console.print(f"Clicando em 'Warning', para andamento do processo...\n")
            btn_no = main_window.child_window(title="Warning")
            if btn_no.exists(timeout=5) and btn_no.is_enabled():
                btn_no.set_focus()
                btn_no.click()
                await worker_sleep(3)
                return {
                    "sucesso": False,
                    "retorno": f"A soma dos pagamentos não bate com o valor da nota, sendo o valor da nota {valor}\n",
                }

        except:
            console.print("Não há warnings durante a inserção de vencimento e valor...\n")

    async def incluir_registro(self, chave_nfe=None, chave_cte=None):
        max_attempts = 3
        i = 0
        while i < max_attempts:
            console.print("Clicando no botão de +...\n")
            try:
                # Incluir registro de Nota Fiscal de Entrada
                pyautogui.click(593, 297)
                await worker_sleep(1)
            except Exception as err:
                console.print(f"Não foi possível incluir o registro: {err}", style="bold red")
            
            await worker_sleep(3)
            i += 1
        
        if chave_nfe is not None:
            await worker_sleep(20)
            console.print("\nVerifica se a nota ja foi lançada...")
            chave_nfe = int(chave_nfe)
            status_nf_emsys = await get_status_nf_emsys(chave_nfe)
            if status_nf_emsys.get("status") == "Lançada":
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno="Nota lançada com sucesso!",
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
        
        if chave_cte is not None:
            await worker_sleep(20)
            console.print("\nVerifica se o CTE ja foi lançada...")
            chave_cte = int(chave_cte)
            status_nf_emsys = await get_status_cte_emsys(chave_cte)
            if status_nf_emsys.get("status") == "Lançada":
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno="CTE lançado com sucesso!",
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
            

    async def get_ultimo_item(self):
        send_keys("^({END})")
        await worker_sleep(2)
        send_keys("+{F10}")
        await worker_sleep(1)
        send_keys("{DOWN 2}")
        await worker_sleep(1)
        send_keys("{ENTER}")
        await worker_sleep(2)
        app = Application().connect(title="Alteração de Item")
        main_window = app["Alteração de Item"]
        main_window.set_focus()
        edit = main_window.child_window(class_name="TDBIEditCode", found_index=0)
        index_ultimo_item = int(edit.window_text())
        try:
            btn_cancelar = main_window.child_window(title="&Cancelar")
            btn_cancelar.click()
        except Exception as error:
            btn_cancelar = main_window.child_window(title="Cancelar")
            btn_cancelar.click()
            console.print(f"Erro ao realizar get_ultimo_item: {error}")
        await worker_sleep(1)
        return index_ultimo_item

    async def verify_max_variation(self):
        try:
            console.print("Iniciando a coleta de dados do grid...\n")
            app = Application(backend="uia").connect(class_name="TFrmTelaSelecao")
            main_window = app.window(class_name="TFrmTelaSelecao")
            grid = main_window.child_window(class_name="TcxGridSite")
            grid.set_focus()
            grid_wrapper = grid.wrapper_object()
            send_keys("^({HOME})")
            await worker_sleep(1)
            data_list = []
            last_content = ""
            repeat_count = 0
            max_repeats = 2
            while True:
                send_keys("^c")
                await worker_sleep(1)
                current_content = pyperclip.paste().strip()
                if not current_content:
                    console.print(
                        "Nenhum conteúdo copiado, encerrando loop.", style="bold red"
                    )
                    break
                item_data = await self.parse_copied_content(current_content)
                data_list.extend(item_data)
                last_content = current_content
                send_keys("{DOWN}")
                await worker_sleep(1)
                send_keys("^c")
                await worker_sleep(1)
                next_content = pyperclip.paste().strip()
                if next_content == last_content:
                    send_keys("{PGDN}")
                    await worker_sleep(1)
                    send_keys("{DOWN}")
                    await worker_sleep(1)
                    send_keys("^c")
                    await worker_sleep(1)
                    next_content = pyperclip.paste().strip()
                    if next_content == last_content:
                        repeat_count += 1
                        if repeat_count >= max_repeats:
                            console.print(
                                "Não há mais itens para processar. Fim do grid alcançado.",
                                style="bold green",
                            )
                            break
                    else:
                        repeat_count = 0
                else:
                    repeat_count = 0

            console.print(f"Dados coletados: {data_list}")

            itens_invalidos = []
            for item in data_list:
                variacao_custo = 20
                if not (
                    item["custo_min"] - variacao_custo <= item["curto"] <= item["custo_max"] + variacao_custo
                ):
                    itens_invalidos.append(item["codigo"])

            if itens_invalidos:
                console.print(
                    "Itens que Ultrapassaram a Variação Máxima de Custo",
                    style="bold yellow",
                )
                console.print(f"Códigos dos itens: {itens_invalidos}")
                send_keys("{ESC}")
                observacao = f"Itens que ultrapassaram a variação máxima de custo: {itens_invalidos}. Processo cancelado."
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=observacao,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
            else:
                console.print(
                    "Todos os itens estão dentro da variação de custo permitida.",
                    style="bold green",
                )
                send_keys("{ENTER}")
                observacao = "Todos os itens estão dentro da variação de custo permitida. Processo concluído com sucesso."

                try:
                    await worker_sleep(2)
                    console.print(
                        "Verificando a existencia de POP-UP de Itens que Ultrapassam a Variação Máxima de Custo ...\n"
                    )
                    itens_variacao_maxima = await is_window_open_by_class(
                        "TFrmTelaSelecao", "TFrmTelaSelecao"
                    )
                    if itens_variacao_maxima["IsOpened"] == True:
                        app = Application().connect(class_name="TFrmTelaSelecao")
                        main_window = app["TFrmTelaSelecao"]
                        send_keys("%o")

                    await worker_sleep(2)
                except:
                    observacao = f"Falha ao clicar em OK no POP-UP de Itens que Ultrapassam a Variação Máxima de Custo."
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=observacao,
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
                
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno=observacao,
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
        except Exception as error:
            ...

    async def parse_copied_content(self, content):
        lines = content.strip().split("\n")
        data_list = []
        i = 0
        while i < len(lines):
            line = lines[i].strip()
            if line.startswith("Código"):
                headers_line = line
                headers = headers_line.split("\t")
                i += 1
                if i < len(lines):
                    data_line = lines[i].strip()
                    data = data_line.split("\t")
                    if len(headers) == len(data):
                        item_dict = dict(zip(headers, data))
                        data_list.append(item_dict)
                    else:
                        console.print(
                            "Número de cabeçalhos e dados não correspondem.",
                            style="bold red",
                        )
                        console.print(f"Cabeçalhos: {headers}")
                        console.print(f"Dados: {data}")
                else:
                    console.print(
                        "Sem linha de dados após cabeçalho.", style="bold red"
                    )
                i += 1
            else:
                i += 1

        final_list = []
        for item in data_list:
            try:
                new_item = {
                    "codigo": int(item["Código"]),
                    "descricao": item["Descrição"],
                    "curto": float(item["R$ Curto"].replace(".", "").replace(",", ".")),
                    "custo_min": float(
                        item["R$ Custo Min."]
                        .replace("worker_automate_hub/utils/utils_nfe_entrada.py.", "")
                        .replace(",", ".")
                    ),
                    "custo_max": float(
                        item["R$ Custo Máx."].replace(".", "").replace(",", ".")
                    ),
                }
                final_list.append(new_item)
            except Exception as e:
                console.print(
                    f"Erro ao processar item: {item}. Erro: {e}", style="bold red"
                )
        return final_list

    async def download_xml(
        self, google_drive_folder_id: str, get_gcp_token: RpaConfiguracao, get_gcp_credentials: RpaConfiguracao, chave_nota: str
    ) -> dict:

        try:
            console.print("Verificando a existência do arquivo no Google Drive...\n")
            chave_nota = f"{chave_nota}.xml"
            gcp_credencial = GetCredsGworkspace(
                token_dict=get_gcp_token.conConfiguracao,
                credentials_dict=get_gcp_credentials.conConfiguracao,
            )
            creds = gcp_credencial.get_creds_gworkspace()

            if not creds:
                error_msg = "Erro ao obter autenticação para o GCP"
                console.print(f"{error_msg}...\n")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=error_msg,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

            # Inicializando o serviço do Google Drive
            drive_service = build("drive", "v3", credentials=creds)

            # Query para procurar o arquivo com o nome da chave da nota
            query = f"'{google_drive_folder_id}' in parents and name contains '{chave_nota}'"
            results = (
                drive_service.files()
                .list(
                    q=query,
                    pageSize=10,  # Reduzindo o número de resultados
                    supportsAllDrives=True,
                    includeItemsFromAllDrives=True,
                    fields="files(id, name)",
                )
                .execute()
            )

            # Verificando se o arquivo foi encontrado
            items = results.get("files", [])

            if not items:
                error_msg = f"Nenhum arquivo com o nome {chave_nota} foi encontrado no Google Drive"
                console.print(error_msg)
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=error_msg,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
                

            # Pegando o primeiro arquivo encontrado
            file_id = items[0]["id"]
            file_name = items[0]["name"]
            console.print(f"Arquivo {file_name} encontrado. Iniciando o download...\n")

            # Definindo o caminho local para salvar o arquivo
            file_path = os.path.join(os.path.expanduser("~"), "Downloads", file_name)

            # Iniciando o download
            request = drive_service.files().get_media(fileId=file_id)
            fh = io.FileIO(file_path, "wb")
            downloader = MediaIoBaseDownload(fh, request)

            done = False
            while not done:
                status, done = downloader.next_chunk()
                console.print(f"Download {int(status.progress() * 100)}% concluído.")

            console.print(
                f"Arquivo {file_name} baixado com sucesso e salvo em {file_path}.\n"
            )
            return {
                "sucesso": True,
                "retorno": f"Arquivo {file_name} baixado com sucesso",
            }

        except Exception as e:
            error_msg = f"Erro ao baixar o arquivo do Google Drive, erro: {e}"
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=error_msg,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

    async def get_xml(self, xml_file):
        try:
            username = getpass.getuser()
            xml_name = f"{xml_file}.xml"
            path_to_xml = f"C:\\Users\\{username}\\Downloads\\{xml_name}"
            pyautogui.click(722, 792)
            await worker_sleep(2)
            pyautogui.write(path_to_xml)
            await worker_sleep(2)
            pyautogui.hotkey("enter")
            await worker_sleep(2)
        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"O xml {xml_name} não foi encontrado em {path_to_xml}",
                status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

    async def delete_xml(self, nfe_key: str) -> None:
        """
        Deletes an XML file for a given NFe key from the user's Downloads folder.

        Args:
            nfe_key: The key of the NFe to delete the XML file for.

        Returns:
            None
        """
        try:
            if not nfe_key:
                raise ValueError("nfe_key não pode ser nulo ou vazio")

            xml_filename = f"{nfe_key}.xml"
            download_folder = os.path.join(os.path.expanduser("~"), "Downloads")
            file_path = os.path.join(download_folder, xml_filename)

            if not os.path.exists(file_path):
                console.print(
                    f"Arquivo {xml_filename} não encontrado em {download_folder}.",
                    style="bold yellow",
                )
                return

            if not os.path.isfile(file_path):
                raise ValueError(f"{file_path} não é um arquivo")

            os.remove(file_path)
            console.print(
                f"Arquivo {xml_filename} deletado com sucesso.", style="bold green"
            )
        except Exception as e:
            console.print(
                f"Erro ao deletar o arquivo {xml_filename}: {str(e)}", style="bold red"
            )
            raise Exception(
                f"Erro ao deletar o arquivo {xml_filename}: {str(e)}"
            ) from e

    async def alterar_nop(self, cfop: str, chave_acesso=None) -> RpaRetornoProcessoDTO | None:
        await self.click_principal()
        console.print("Alterando a NOP...\n")
        pyautogui.click(945, 543)
        await worker_sleep(2)
        
        if chave_acesso is not None:
            console.print("\nVerifica se a nota ja foi lançada...")
            nf_chave_acesso = int(chave_acesso)
            status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
            if status_nf_emsys.get("status") == "Lançada":
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno="Nota lançada com sucesso!",
                    status=RpaHistoricoStatusEnum.Sucesso,
                )

        try:
            if self.apenas_isqueiros:
                nop_value = "1102 - COMPRA DE MERCADORIA ADQ. TERCEIROS - 1.102"
                console.print(f"Inserindo a informação da NOP para apenas isqueiros: {nop_value} ...\n")
                await worker_sleep(5)
                set_combobox("||List", nop_value)
                await worker_sleep(2)
                pyautogui.hotkey("enter")
                await worker_sleep(6)
                await self.incluir_registro()
            else:
                if cfop:
                    console.print(f"Inserindo a informação da NOP, caso se aplique {cfop} ...\n")
                    if cfop not in ["5910", "6910"]:
                        await worker_sleep(5)
                        set_combobox("||List", "1403 - COMPRA DE MERCADORIA - 1.403")
                        await worker_sleep(2)
                        pyautogui.hotkey("enter")
                        await worker_sleep(6)
                        await self.incluir_registro()
                        await worker_sleep(15)
                        
                        if chave_acesso is not None:
                            console.print("\nVerifica se a nota ja foi lançada...")
                            nf_chave_acesso = int(chave_acesso)
                            status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
                            if status_nf_emsys.get("status") != "Lançada":
                                raise ValueError("A nota não possui itens com o mesmo CFOP de capa. Necessário que ao menos um item possua este CFOP")
                                
                    else:
                        pyautogui.hotkey("esc")
                        observacao = f"Nota bonificada do tipo ({cfop}), está retornando diferença de itens sem o mesmo CFOP de capa. Por favor verifique a nota."
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno=observacao,
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                        )
        except ValueError as e:
            console.print(f"Erro de validação ao alterar o NOP: {e}", style="bold red")
            raise
        except Exception as e:
            console.print(f"Erro inesperado ao alterar o NOP: {e}", style="bold red")
            raise

        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno="Nota lançada com sucesso!",
            status=RpaHistoricoStatusEnum.Sucesso,
        )

    async def verify_nf_incuded(self) -> bool:
        try:
            nota_incluida = pyautogui.locateOnScreen(
                ASSETS_PATH + "\\entrada_notas\\nota_fiscal_incluida.png", confidence=0.7
            )
            if nota_incluida:
                return True
            else:
                return False
        except Exception as e:
            console.print(f"Error: {e}")
            return False


from typing import Optional
from pydantic import BaseModel, Field


class ConfigEntradaSAP(BaseModel):
    user: str
    password: str
    empresa: str
    historico_id: Optional[str] = None

    def get(self, key, default=None):
        return getattr(self, key, default)

class RpaProcessoSapDTO(BaseModel):
    configEntrada: ConfigEntradaSAP
    uuidProcesso: str = Field(..., alias="uuidProcesso")



import asyncio
import getpass
from datetime import datetime

from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from rich.console import Console
import re
import os
from pywinauto.keyboard import send_keys
from worker_automate_hub.utils.util import login_emsys
import warnings
from pywinauto.application import Application
from worker_automate_hub.api.client import get_config_by_name, get_status_cte_emsys
from worker_automate_hub.api.ahead_service import save_xml_to_downloads
from worker_automate_hub.utils.util import (
    kill_all_emsys,
    delete_xml,
    set_variable,
    type_text_into_field,
    worker_sleep,
)
from pywinauto_recorder.player import set_combobox

from datetime import timedelta
import pyautogui
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.utils_nfe_entrada import EMSys

ASSETS_BASE_PATH = "assets/cte_xml/"
emsys = EMSys()

console = Console()
pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False


async def click_importar_xml():
    await worker_sleep(5)
    try:
        pyautogui.click(1190, 248)
    except Exception as e:
        # console.print(f"Erro ao conectar a janela {title}")
        raise Exception(f"Erro ao conectar a janela: {e}")


async def importar_xml_conhecimento():

    try:
        await worker_sleep(5)
        app = Application(backend="win32").connect(
            title="Importar XML Conhecimento", found_index=0
        )
        main_window = app["Importar XML Conhecimento"]

        # Clicar em aquisicao de servico de transporte D/E
        aquisicao_servico = main_window.child_window(
            class_name="TDBIComboBox", found_index=1
        )
        aquisicao_servico.select("1353 - AQUISICAO DE SERVICO DE TRANSPORTE D/E")

        await worker_sleep(2)

        # Clicar em rodoviário
        rodoviario = main_window.child_window(class_name="TDBIComboBox", found_index=0)
        rodoviario.select("RODOVIARIO")

        await worker_sleep(2)

        # Clicar em OK
        max_tentativas = 3
        for tentativa in range(max_tentativas):
            try:
                botao = main_window.child_window(class_name="TBitBtn", found_index=1)
                botao.wait("enabled", timeout=5)
                botao.click()

                await worker_sleep(1)

                janela_filha = botao.top_level_parent()
                if not janela_filha.exists(timeout=2):
                    print("Janela filha fechada com sucesso.")
                    break

            except Exception as e:
                print(f"Erro na tentativa {tentativa + 1}: {e}")
        else:
            print("A janela não foi fechada após várias tentativas.")

    except Exception as e:
        # console.print(f"Erro ao conectar a janela {window_title}")
        raise Exception(f"Erro ao conectar a janela: {e}")


async def selecionar_xml(cte):
    await worker_sleep(5)
    nota = cte["chaveCte"]
    nota_cte = f"{nota}.xml"
    username = getpass.getuser()
    path_to_xml = f"C:\\Users\\{username}\\Downloads\\{nota_cte}"
    app = Application(backend="win32").connect(title="Abrir", found_index=0)
    main_window = app["Abrir"]

    await worker_sleep(2)

    #  Selecionar nota baixada
    send_keys(path_to_xml)

    await worker_sleep(3)

    # Tenta clicar no botão abrir apos digitar caminho do XML
    max_tentativas = 3
    for tentativa in range(max_tentativas):
        try:
            botao = main_window.child_window(class_name="Button", found_index=0)
            botao.wait("enabled", timeout=5)
            botao.click()

            await worker_sleep(1)

            janela_filha = botao.top_level_parent()
            if not janela_filha.exists(timeout=2):
                print("Janela filha fechada com sucesso.")
                break

        except Exception as e:
            print(f"Erro na tentativa {tentativa + 1}: {e}")
    else:
        print("A janela não foi fechada após várias tentativas.")


def calcular_vencimento_333(data_emissao):

    data_emissao = datetime.strptime(data_emissao, "%d/%m/%Y")

    if 23 <= data_emissao.day or data_emissao.day <= 8:
        faturamento = datetime(data_emissao.year, data_emissao.month, 9)
        if data_emissao.day >= 23:
            faturamento = faturamento + timedelta(days=31)
            faturamento = faturamento.replace(day=9)
    else:
        faturamento = datetime(data_emissao.year, data_emissao.month, 23)
    if faturamento.weekday() == 5:
        faturamento -= timedelta(days=1)
    elif faturamento.weekday() == 6:
        faturamento -= timedelta(days=2)

    vencimento = faturamento + timedelta(days=15)

    if vencimento.weekday() == 5:
        vencimento -= timedelta(days=1)
    elif vencimento.weekday() == 6:
        vencimento -= timedelta(days=2)

    # Ajuste para vencimentos com data anterior ao dia atual
    hoje = datetime.now()
    if vencimento.date() < hoje.date():
        vencimento = hoje + timedelta(days=1)

    return vencimento.strftime("%d/%m/%Y")


def calcular_vencimento_1353(data_emissao):
    data_emissao = datetime.strptime(data_emissao, "%d/%m/%Y")

    if data_emissao.day <= 15:
        mes_vencimento = data_emissao.month + 1
        ano_vencimento = data_emissao.year
        if mes_vencimento > 12:
            mes_vencimento = 1
            ano_vencimento += 1
        vencimento = datetime(ano_vencimento, mes_vencimento, 1)
    else:
        mes_vencimento = data_emissao.month + 1
        ano_vencimento = data_emissao.year
        if mes_vencimento > 12:
            mes_vencimento = 1
            ano_vencimento += 1
        vencimento = datetime(ano_vencimento, mes_vencimento, 16)

    # Ajuste para vencimentos com data anterior ao dia atual
    hoje = datetime.now()
    if vencimento.date() < hoje.date():
        vencimento = hoje + timedelta(days=1)

    return vencimento.strftime("%d/%m/%Y")


async def janela_conhecimento_frete(cte):
    await worker_sleep(3)
    app = Application(backend="win32").connect(
        title="Conhecimento de Frete", found_index=0
    )
    main_window = app["Conhecimento de Frete"]
    despesa = await get_config_by_name("CTE_Despesas")
    cnpj_emitente = cte["cnpjEmitente"]
    tipo_desp = despesa.conConfiguracao["despesas"]
    valor_frete = cte["valorFrete"]
    codigo_despesa = 83  # Valor padrão
    data_emissao = cte["dataEmissao"]
    natureza = cte["natureza"]
    for item in tipo_desp:
        for chave in ["emitente", "empresas"]:
            if chave in item:
                for empresa in item[chave]:
                    if empresa["cnpj"] == cnpj_emitente:
                        codigo_despesa = item["codigoDespesa"]
                        break
                else:
                    continue  # continua no próximo grupo se não achou
                break  # sai do loop interno se achou
        else:
            continue
        break  # sai do loop externo se achou

    print(codigo_despesa)

    # Selecionar natureza da operação 9 - outros
    select_other = main_window.child_window(
        class_name="TDBIComboBoxValues", found_index=1
    )
    select_other.select("9 - Outros")

    await worker_sleep(2)

    # Preencher Tipo de despesa
    campo_tipo_despesa = main_window.child_window(
        class_name="TDBIEditCode", found_index=2
    ).click()
    await worker_sleep(2)
    type_text_into_field(
        text=codigo_despesa,
        field=campo_tipo_despesa,
        empty_before=True,
        chars_to_empty="3",
    )
    campo_tipo_despesa.type_keys("{TAB}")

    # Clicar em valores
    try:
        pyautogui.click(602, 297)
        await worker_sleep(5)

    except Exception as e:
        raise Exception(f"Erro ao clicar em Valores: {e}")

    # Preencher Base ICMS não trib
    base_icms = main_window.child_window(
        class_name="TDBIEditNumber", found_index=20
    ).click()
    base_icms.type_keys("{DEL}")
    base_icms.type_keys(valor_frete)

    # Preencher Valor frete
    base_icms = main_window.child_window(
        class_name="TDBIEditNumber", found_index=14
    ).click()
    for _ in range(10):
        base_icms.type_keys("{DEL}")
    base_icms.type_keys(valor_frete)

    # Clicar em Pagamento
    try:
        pyautogui.click(610, 313)
        await worker_sleep(5)

    except Exception as e:
        raise Exception(f"Erro ao clicar em Valores: {e}")

    # Clicar em Parcelado
    try:
        pyautogui.click(1296, 460)
        await worker_sleep(5)

    except Exception as e:
        raise Exception(f"Erro ao clicar em Valores: {e}")

    # Selecionar tipo de cobrança
    tipo_cobranca = main_window.child_window(class_name="TDBIComboBox", found_index=0)
    tipo_cobranca.select("BANCO DO BRASIL BOLETO")

    if natureza == "1353":
        vencimento = calcular_vencimento_1353(data_emissao)
    elif natureza == "333":
        vencimento = calcular_vencimento_333(data_emissao)

    try:
        # Clicar no botão - para apagar registro se existir
        main_window.child_window(class_name="TDBIBitBtn", found_index=4).click()

        await worker_sleep(2)

        # Clicar em sim para excluir registro
        pyautogui.click(953, 601)
        await worker_sleep(2)

        app = Application(backend="win32").connect(title="Informação", found_index=0)
        main_window = app["Informação"]

        # Clicar em OK
        main_window.child_window(title="OK", found_index=0).click()
        await worker_sleep(2)
    except:
        pass

    # Voltar pra janela conhecimento de frete
    app = Application(backend="win32").connect(
        title="Conhecimento de Frete", found_index=0
    )
    main_window = app["Conhecimento de Frete"]

    # Inserir data de vencimento
    data_vencimento = main_window.child_window(
        class_name="TDBIEditDate", found_index=0
    ).click()
    await worker_sleep(2)
    type_text_into_field(vencimento, data_vencimento, True, "2")

    # clicar no botao de inserir o valor
    btn_inserir_valor = main_window.child_window(
        class_name="TBitBtn", found_index=0
    ).click()

    # Clicar no botão + para salvar
    btn_salvar = main_window.child_window(
        class_name="TDBIBitBtn", found_index=5
    ).click()

    # Clicar no botão + para salvar tudo(aguarda dados de rateio)
    try:
        pyautogui.click(581, 252)
        await worker_sleep(3)

    except Exception as e:
        raise Exception(f"Erro ao clicar em Valores: {e}")


async def janela_information():
    await worker_sleep(3)
    app = Application(backend="win32").connect(title="Information", found_index=0)
    main_window = app["Information"]

    # Clicar em OK
    main_window.child_window(title="OK", found_index=0).click()


async def desmarcar_flag():
    app = Application(backend="win32").connect(
        title="Conhecimento de Frete", found_index=0
    )
    main_window = app["Conhecimento de Frete"]

    # Clicar em Informar nota no conhecimento
    main_window.child_window(class_name="TCheckBox", found_index=0).click()

    # Clicar no botão + para salvar tudo(aguarda dados de rateio)
    try:
        pyautogui.click(581, 252)
        await worker_sleep(5)

    except Exception as e:
        raise Exception(f"Erro ao clicar em Valores: {e}")

    await worker_sleep(3)
    try:
        app = Application(backend="win32").connect(title="Informação", found_index=0)
        main_window = app["Informação"]

        # Clicar em OK
        main_window.child_window(title="OK", found_index=0).click()
    except:
        pass


async def importar_cte_xml(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    try:
        config = await get_config_by_name("login_emsys")
        cte = task.configEntrada

        # Verifica se a nota já foi lançada
        status_nf_emsys = await get_status_cte_emsys(cte["chaveCte"])
        status = status_nf_emsys["status"]
        print(f"Status da nota:  {status}")
        if status == "Lançada":
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Nota: {cte['chaveCte']} já foi lançada",
                status=RpaHistoricoStatusEnum.Descartado,
            )

        await kill_all_emsys()

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3_35.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field(
                "Conhecimento de Frete", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(2)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(6)

        # Download XML
        console.log("Realizando o download do XML..\n")
        await save_xml_to_downloads(cte["chaveCte"])
        await worker_sleep(5)
        await click_importar_xml()
        await importar_xml_conhecimento()
        await selecionar_xml(cte)
        await janela_conhecimento_frete(cte)
        await worker_sleep(8)
        try:
            await janela_information()
        except:
            pass
        await desmarcar_flag()

        # Verifica se a nota foi lançada e atualizado no banco
        status_nf_emsys = await get_status_cte_emsys(cte["chaveCte"])
        status = status_nf_emsys["status"]
        print(f"Status da nota:  {status}")
        if status != "Lançada":
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Nota: {cte['chaveCte']} não foi lançada com sucesso",
                status=RpaHistoricoStatusEnum.Falha,
            )
        else:
            return RpaRetornoProcessoDTO(
                sucesso=True,
                retorno=f"Suceso no processo CTE com XML",
                status=RpaHistoricoStatusEnum.Sucesso,
            )

    except Exception as e:
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro no processo CTE com XML: {e}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )
    finally:
        await delete_xml(cte.get("chaveCte"))
